import json
import unittest

from dashio import Chart, ChartLine
from dashio.iotcontrol.enums import Color, ChartLineType, ChartXAxisLabelsStyle


class TestChart(unittest.TestCase):
    def _get_cfg_dict(self, cfg_list: list):
        json_str = cfg_list[0].rpartition('\t')[2]
        return json.loads(json_str)

    def test_graph_line(self):
        test_data = ChartLine("GRAPHLINE", line_type=ChartLineType.LINE, color=Color.BLACK)
        test_str_list = test_data.get_line_data().split('\t')
        self.assertEqual(ChartLineType(test_str_list[2]), ChartLineType.LINE, "line type should be LINE")
        self.assertEqual(Color(int(test_str_list[3])), Color.BLACK, "Color should be BLACK")

    def test_graph_control_type(self):
        test_control = Chart("CHARTID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[2], 'CHRT', "control type should be GRPH")

    def test_graph_control_id(self):
        test_control = Chart("CHARTID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[3], 'CHARTID', "control_id type should be CHARTID")

    def test_graph_x_axis_label(self):
        test_control = Chart("CHARTID", x_axis_label="XAXISLABEL")
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['xAxisLabel'], 'XAXISLABEL', "CFG xAxisLabel Should be 'XAXISLABEL'")

    def test_graph_y_axis_label(self):
        test_control = Chart("CHARTID", y_axis_label="YAXISLABEL")
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['yAxisLabel'], 'YAXISLABEL', "CFG yAxisLabel Should be 'YAXISLABEL'")

    def test_graph_x_axis_min(self):
        test_control = Chart("CHARTID", x_axis_min=1.0)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['xAxisMin'], 1.0, "CFG xAxisMin Should be 1.0")

    def test_graph_y_axis_min(self):
        test_control = Chart("CHARTID", y_axis_min=1.0)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['yAxisMin'], 1.0, "CFG yAxisMin Should be 1.0")

    def test_graph_x_axis_max(self):
        test_control = Chart("CHARTID", x_axis_max=1.0)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['xAxisMax'], 1.0, "CFG xAxisMax Should be 1.0")

    def test_graph_y_axis_max(self):
        test_control = Chart("CHARTID", y_axis_max=1.0)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['yAxisMax'], 1.0, "CFG yAxisMax Should be 1.0")

    def test_graph_x_axis_num_bars(self):
        test_control = Chart("CHARTID", x_axis_num_bars=5)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['xAxisNumBars'], 5, "CFG xAxisNumBars Should be 5")

    def test_graph_x_axis_labels_style(self):
        test_control = Chart("CHARTID", x_axis_labels_style=ChartXAxisLabelsStyle.BETWEEN)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(ChartXAxisLabelsStyle(cfg_dict['xAxisLabelsStyle']), ChartXAxisLabelsStyle.BETWEEN, "CFG xAxisLabelsStyle Should be BETWEEN")


if __name__ == '__main__':
    unittest.main()



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from .device import Device
from .tcpconnection import TCPConnection
from .mqttconnection import MQTTConnection
from .zmqconnection import ZMQConnection
from .dashconnection import DashConnection
from .serialconnection import SerialConnection
from .load_config import decode_cfg64, encode_cfg64, load_all_controls_from_config, get_control_dict_from_config, get_control_from_config
# from .bleconnection import BLEConnection
from .iotcontrol.enums import (
    Color,
    Icon,
    Precision,
    SoundName,
    Keyboard,
    TextAlignment,
    TitlePosition,
    SliderBarStyle,
    DialNumberPosition,
    DialStyle,
    ChartLineType,
    TimeGraphLineType,
    TimeGraphPositionOfKey,
    ButtonState,
    LabelStyle,
    KnobStyle,
    ChartXAxisLabelsStyle,
    TextFormat,
    DirectionStyle,
    ColorPickerStyle,
    ControlName
)
from .iotcontrol.audio_visual_display import AudioVisualDisplay
from .iotcontrol.chart import Chart, ChartLine
from .iotcontrol.slider import Slider
from .iotcontrol.textbox import TextBox
from .iotcontrol.button import Button
from .iotcontrol.time_graph import TimeGraph, TimeGraphLine, DataPoint
from .iotcontrol.knob import Knob
from .iotcontrol.dial import Dial
from .iotcontrol.direction import Direction
from .iotcontrol.map import Map, MapLocation
from .iotcontrol.alarm import Alarm
from .iotcontrol.menu import Menu
from .iotcontrol.selector import Selector
from .iotcontrol.label import Label
from .iotcontrol.device_view import DeviceView
from .iotcontrol.control import Control, ControlPosition
from .iotcontrol.button_group import ButtonGroup
from .iotcontrol.event_log import EventData, EventLog
from .iotcontrol.color_picker import ColorPicker



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import logging
import socket
import threading
import json
import shortuuid
import zmq
from zeroconf import IPVersion, ServiceInfo, Zeroconf

from . import ip
from .constants import CONNECTION_PUB_URL


class ZMQConnection(threading.Thread):
    """Setups and manages a connection thread to iotdashboard via ZMQ."""

    def _zconf_publish_zmq(self, sub_port, pub_port):
        zconf_desc = {
            'subPort': str(sub_port),
            'pubPort': str(pub_port),
            'deviceID': ','.join(self.device_id_list)
        }

        zconf_info = ServiceInfo(
            "_DashZMQ._tcp.local.",
            f"{self.zmq_connection_uuid}._DashZMQ._tcp.local.",
            addresses=[socket.inet_aton(self.local_ip)],
            port=pub_port,
            properties=zconf_desc,
            server=self.host_name + ".",
        )
        self.zeroconf.update_service(zconf_info)

    def add_device(self, device):
        """Add a Device to the connextion

        Parameters
        ----------
            device (Device):
                The Device to add.
        """
        if device.device_id not in self._device_id_list:
            self._device_id_list.append(device.device_id)
            device.register_connection(self)
            self._send_dash_announce()

    def _send_dash_announce(self):
        msg = {
            'msgType': 'send_announce',
            'connectionUUID': self.zmq_connection_uuid
        }
        logging.debug("ZMQ SEND ANNOUNCE: %s", msg)
        self.tx_zmq_pub.send_multipart([b"COMMAND", json.dumps(msg).encode()])

    def close(self):
        """Close the connection."""

        self.zeroconf.unregister_all_services()
        self.zeroconf.close()
        self.running = False

    def __init__(self, zmq_out_url="*", pub_port=5555, sub_port=5556, context: zmq.Context = None):
        """ZMQConnection

        Parameters
        ---------
            zmq_out_url (str, optional):
                URL to use. Defaults to "*".
            pub_port (int, optional):
                Port to publish with. Defaults to 5555.
            sub_port (int, optional):
                Port to subscribe with. Defaults to 5556.
            context (ZMQ Context, optional): Defaults to None.
        """

        threading.Thread.__init__(self, daemon=True)
        self.context = context or zmq.Context.instance()
        self.running = True

        self._device_id_list = []
        self.zmq_connection_uuid = "ZMQ:" + shortuuid.uuid()
        self.b_zmq_connection_id = self.zmq_connection_uuid.encode('utf-8')

        host_name = socket.gethostname()
        host_list = host_name.split(".")
        # rename for .local mDNS advertising
        self.host_name = f"{host_list[0]}.local"

        self.local_ip = ip.get_local_ip_address()
        self.zeroconf = Zeroconf(ip_version=IPVersion.V4Only)
        self._zconf_publish_zmq(sub_port, pub_port)
        self.start()

    def _add_device_rx(self, msg_dict):
        """Connect to another device"""
        device_id = msg_dict["deviceID"]
        logging.debug("ZMQ DEVICE CONNECT: %s", device_id)
        #  TODO finish this

    def _del_device_rx(self, msg_dict):
        device_id = msg_dict["deviceID"]
        logging.debug("TCP DEVICE_DISCONNECT: %s", device_id)
        #  TODO finish this

    def _zmq_command(self, msg_dict: dict):
        logging.debug("TCP CMD: %s", msg_dict)
        if msg_dict['msgType'] == 'connect':
            self._add_device_rx(msg_dict)
        if msg_dict['msgType'] == 'disconnect':
            self._del_device_rx(msg_dict)

    def run(self):

        self.tx_zmq_pub = self.context.socket(zmq.PUB)
        self.tx_zmq_pub.bind(CONNECTION_PUB_URL.format(id=self.zmq_connection_uuid))

        #  Subscribe on ALL, and my connection
        self.rx_zmq_sub = self.context.socket(zmq.SUB)
        # Subscribe on ALL, and my connection
        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "ALL")
        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "COMMAND")
        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "MQTT")
        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, self.zmq_connection_uuid)
        # rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "ANNOUNCE")

        ext_tx_zmq_pub = self.context.socket(zmq.PUB)
        ext_tx_zmq_pub.bind(self.tx_url_external)

        self.ext_rx_zmq_sub = self.context.socket(zmq.SUB)
        self.ext_rx_zmq_sub.bind(self.rx_url_external)

        # Subscribe on WHO, and my deviceID
        self.ext_rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b'\tWHO')

        poller = zmq.Poller()
        poller.register(self.ext_rx_zmq_sub, zmq.POLLIN)
        poller.register(self.rx_zmq_sub, zmq.POLLIN)

        while self.running:
            try:
                socks = dict(poller.poll(50))
            except zmq.error.ContextTerminated:
                break
            if self.ext_rx_zmq_sub in socks:
                message = self.ext_rx_zmq_sub.recv()
                logging.debug("ZMQ Rx: %s", message.decode('utf-8').rstrip())
                self.tx_zmq_pub.send_multipart([message, self.b_zmq_connection_id])

            if self.rx_zmq_sub in socks:
                [msg_to, data] = self.rx_zmq_sub.recv_multipart()

                if msg_to == b'ALL':
                    logging.debug("ZMQ Tx: %s", data.decode('utf-8').rstrip())
                    ext_tx_zmq_pub.send(data)
                elif msg_to == b'COMMAND':
                    self._zmq_command(json.loads(data))
                    return

        self.tx_zmq_pub.close()
        self.rx_zmq_sub.close()



import json
import unittest


def _get_cfg_dict(cfg_list: list):
    json_str = cfg_list[0].rpartition('\t')[2]
    return json.loads(json_str)


"""
class TestMapLocation(unittest.TestCase):

    def test_map_location_timestamp(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748)
        test_loc = json.loads(str(test_location))
        self.assertIsInstance(dateutil.parser.isoparse(test_loc['time']), datetime, "Should be datetime")

    def test_map_location_message(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748)
        test_loc = json.loads(str(test_location))
        self.assertEqual(test_loc['message'], 'TAG', "message should be TAG")

    def test_map_location_latitude(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748)
        test_loc = json.loads(str(test_location))
        self.assertEqual(test_loc['latitude'], -45.237101516008835, "latitude should be -45.237101516008835")

    def test_map_location_longitude(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748)
        test_loc = json.loads(str(test_location))
        self.assertEqual(test_loc['longitude'], 168.84818243505748, "latitude should be 168.84818243505748")

    def test_map_location_avg_speed(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748, average_speed=10)
        test_loc = json.loads(str(test_location))
        self.assertEqual(test_loc['avgeSpeed'], 10, "avgeSpeed should be 10")

    def test_map_location_peak_speed(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748, peak_speed=10)
        test_loc = json.loads(str(test_location))
        self.assertEqual(test_loc['peakSpeed'], 10, "peakSpeed should be 10")

    def test_map_location_course(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748, course=10)
        test_loc = json.loads(str(test_location))
        self.assertEqual(test_loc['course'], 10, "course should be 10")

    def test_map_location_altitude(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748, altitude=10)
        test_loc = json.loads(str(test_location))
        self.assertEqual(test_loc['altitude'], 10, "altitude should be 10")

    def test_map_location_distance(self):
        test_location = MapLocation("TAG", -45.237101516008835, 168.84818243505748, distance=10)
        test_loc = json.loads(str(test_location))
        self.assertEqual(test_loc['distance'], 10, "distance should be 10")


class TestMap(unittest.TestCase):

    def test_knob_control_type(self):
        test_control = Map("MAPID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[2], 'MAP', "control type should be MAP")

    def test_knob_control_id(self):
        test_control = Map("MAPID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[3], 'MAPID', "control type should be MAPID")

"""


if __name__ == '__main__':
    unittest.main()



import json
import unittest


class TestControl(unittest.TestCase):
    def _get_cfg_dict(self, cfg_str):
        json_str = cfg_str.rpartition('\t')[2]
        return json.loads(json_str)


if __name__ == '__main__':
    unittest.main()



import json
import unittest

from dashio import ButtonGroup


def _get_cfg_dict(cfg_str):
    json_str = cfg_str.rpartition('\t')[2]
    return json.loads(json_str)


class TestButtonGroup(unittest.TestCase):

    def test_button_group_control_type(self):
        test_control = ButtonGroup("BGROUPID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[2], 'BTGP', "control type should be BTGP")

    def test_button_group_control_id(self):
        test_control = ButtonGroup("BGROUPID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[3], 'BGROUPID', "control type should be BGROUPID")


if __name__ == '__main__':
    unittest.main()



import json
import unittest

from dashio import Label
from dashio.iotcontrol.enums import Color, LabelStyle


class TestLabel(unittest.TestCase):
    def _get_cfg_dict(self, cfg_list: list):
        json_str = cfg_list[0].rpartition('\t')[2]
        return json.loads(json_str)

    def test_label_control_type(self):
        test_control = Label("LABELID", color=Color.BLUE)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(Color(int(cfg_dict['color'])), Color.BLUE, "CFG yAxisMax Should be 1.0")

    def test_label_control_id(self):
        test_control = Label("LABELID", style=LabelStyle.BORDER)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(LabelStyle(cfg_dict['style']), LabelStyle.BORDER, "CFG style Should be 1.0")


if __name__ == '__main__':
    unittest.main()



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import json
import logging
import ssl
import threading
import time

import paho.mqtt.client as mqtt
import shortuuid
import zmq

from .constants import CONNECTION_PUB_URL


class DashControl():
    """Class to stare dash connection info
    """
    def get_state(self) -> str:
        """Not used by this class as its a CFG only control
        """
        return ""

    def get_cfg(self, data: list) -> str:
        """Returns the CFG info for the Dash connection. Called by iotdashboard app

        Parameters
        ----------
        data:
            List sent by dashboard app.

        Returns
        -------
        str
            The CFG string
        """
        try:
            # num_columns = data[3]
            dashboard_id = data[2]
        except IndexError:
            return ""
        cfg_str = f"\tCFG\t{dashboard_id}\t" + self.cntrl_type + "\t" + json.dumps(self._cfg) + "\n"
        return cfg_str

    def get_cfg64(self, data: list) -> dict:
        """Returns the CFG dict for this TCP control

        Returns
        -------
        dict
            The CFG string for this control
        """
        return self._cfg

    def __init__(self, control_id, username="", servername=""):
        self.cntrl_type = "DASHIO"
        self._cfg = {}
        self._cfg["controlID"] = control_id
        self.control_id = control_id
        self.username = username
        self.servername = servername

    @property
    def username(self) -> str:
        """username

        Returns
        -------
        str
            The username for the current connection
        """
        return self._cfg["userName"]

    @username.setter
    def username(self, val: str):
        self._cfg["userName"] = val

    @property
    def servername(self) -> str:
        """servername

        Returns
        -------
        str
            The servername for the current connection
        """
        return self._cfg["hostURL"]

    @servername.setter
    def servername(self, val: str):
        self._cfg["hostURL"] = val


class DashConnection(threading.Thread):
    """Setups and manages a connection thread to the Dash Server.

    Attributes
    ----------
    dash_control : Dash
        A Dash control

    Methods
    -------
    add_device(Device) :
        add a Deive to the connection
    set_connection(username, password):
        change the connection username and password
    close() :
        close the connection
    """
    def _on_connect(self, client, userdata, flags, msg):
        if msg == 0:
            self._connected = True
            self._disconnected = False
            for device_id in self._device_id_list:
                control_topic = f"{self.username}/{device_id}/control"
                self._dash_c.subscribe(control_topic, 0)
            for device_id in self._device_id_rx_list:
                data_topic = f"{self.username}/{device_id}/data"
                self._dash_c.subscribe(data_topic, 0)
            self._send_dash_announce()
            logging.debug("connected OK")
        else:
            logging.debug("Bad connection Returned code=%s", msg)

    def _on_disconnect(self, client, userdata, msg):
        logging.debug("disconnecting reason  %s", msg)
        self._connected = False
        self._disconnected = True
        self.disconnect_timeout = 1.0

    def _on_message(self, client, obj, msg):
        data = str(msg.payload, "utf-8").strip()
        logging.debug("DASH RX:\n%s", data)
        self.tx_zmq_pub.send_multipart([msg.payload, self._b_zmq_connection_uuid])

    def _on_subscribe(self, client, obj, mid, granted_qos):
        logging.debug("Subscribed: %s %s", str(mid), str(granted_qos))

    def _on_log(self, client, obj, level, string):
        logging.debug(string)

    def add_device(self, device):
        """Add a Device to the connextion

        Parameters
        ----------
            device (Device):
                The Device to add.
        """
        if device.device_id not in self._device_id_list:
            self._device_id_list.append(device.device_id)
            device.register_connection(self)
            if self._connected:
                control_topic = f"{self.username}/{device.device_id}/control"
                self._dash_c.subscribe(control_topic, 0)
                self._send_dash_announce()

    def _add_device_rx(self, msg_dict):
        """Connect to another device"""
        device_id = msg_dict["deviceID"]
        logging.debug("DASH DEVICE CONNECT: %s", device_id)
        if device_id not in self._device_id_rx_list:
            self._device_id_rx_list.append(device_id)
            data_topic = f"{self.username}/{device_id}/data"
            self._dash_c.subscribe(data_topic, 0)

    def _del_device_rx(self, msg_dict):
        device_id = msg_dict["deviceID"]
        if device_id in self._device_id_rx_list:
            data_topic = f"{self.username}/{device_id}/data"
            self._dash_c.unsubscribe(data_topic)
            logging.debug("DASH DEVICE_DISCONNECT: %s", device_id)
            del self._device_id_rx_list[device_id]

    def _send_dash_announce(self):
        msg = {
            'msgType': 'send_announce',
            'connectionUUID': self.zmq_connection_uuid
        }
        logging.debug("DASH SEND ANNOUNCE: %s", msg)
        self.tx_zmq_pub.send_multipart([b"COMMAND", json.dumps(msg).encode()])

    def set_connection(self, username: str, password: str):
        """Changes the connection to the DashIO server

        Parameters
        ----------
        username : str
            username for the server
        password : str
            password for the server
        """
        self._dash_c.disconnect()
        self.username = username
        self._dash_c.username_pw_set(username, password)
        self._dash_c.connect(self.host, self.port)

    def __init__(
        self,
        username="",
        password="",
        host='dash.dashio.io',
        port=8883,
        use_ssl=True,
        context: zmq.Context = None
    ):
        """
        Setups and manages a connection thread to the Dash Server.

        Parameters
        ---------
            host : str
                The server name of the dash host.
            port : int
                Port number to connect to. Defaults to 8883.
            username : str
                username for the dash connection.
            password : str
                password for the dash connection.
            use_ssl : Boolean
                Defaults to True.
        """

        threading.Thread.__init__(self, daemon=True)

        self.context = context or zmq.Context.instance()
        self._connected = False
        self._disconnected = True
        self.zmq_connection_uuid = "DASH:" + shortuuid.uuid()
        self._b_zmq_connection_uuid = self.zmq_connection_uuid.encode('utf-8')
        self._device_id_list = []
        self._device_id_rx_list = []
        # self.LWD = "OFFLINE"
        self.running = True
        self.username = username
        self.host = host
        self.port = port
        self._dash_c = mqtt.Client()
        # Assign event callbacks
        self._dash_c.on_message = self._on_message
        self._dash_c.on_connect = self._on_connect
        self._dash_c.on_disconnect = self._on_disconnect
        # self.dash_c.on_publish = self.__on_publish
        self._dash_c.on_subscribe = self._on_subscribe
        # self.connection_control = DashControl(self.zmq_connection_uuid, username, host)
        if use_ssl:
            self._dash_c.tls_set(
                ca_certs=None,
                certfile=None,
                keyfile=None,
                cert_reqs=ssl.CERT_REQUIRED,
                tls_version=ssl.PROTOCOL_TLSv1_2,
                ciphers=None,
            )
            self._dash_c.tls_insecure_set(False)

        # self.dash_c.on_log = self.__on_log
        # self.dash_c.will_set(self.data_topic, self.LWD, qos=1, retain=False)
        # Connect
        if username and password:
            self._dash_c.username_pw_set(username, password)
            try:
                self._dash_c.connect(host, port)
            except mqtt.socket.gaierror as error:
                logging.debug("No connection to internet: %s", str(error))
        # Start subscribe, with QoS level 0
        self.rx_zmq_sub = self.context.socket(zmq.SUB)
        self.disconnect_timeout = 15.0
        self.start()

    def close(self):
        """Close the connection."""
        self.running = False

    def _dash_command(self, msg_dict: dict):
        logging.debug("TCP CMD: %s", msg_dict)
        if msg_dict['msgType'] == 'connect':
            self._add_device_rx(msg_dict)
        if msg_dict['msgType'] == 'disconnect':
            self._del_device_rx(msg_dict)

    def run(self):
        self._dash_c.loop_start()

        self.tx_zmq_pub = self.context.socket(zmq.PUB)
        self.tx_zmq_pub.bind(CONNECTION_PUB_URL.format(id=self.zmq_connection_uuid))

        #  Subscribe on ALL, and my connection
        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "ALL")
        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "DASH")
        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, self.zmq_connection_uuid)
        poller = zmq.Poller()
        poller.register(self.rx_zmq_sub, zmq.POLLIN)
        data_topic = ""
        control_type = ""
        while self.running:
            try:
                socks = dict(poller.poll(15))
            except zmq.error.ContextTerminated:
                break

            if self.rx_zmq_sub in socks:
                try:
                    [msg_to, data] = self.rx_zmq_sub.recv_multipart()
                except ValueError:
                    logging.debug("DASH value error")
                    continue
                if not data:
                    logging.debug("DASH no data error")
                    continue
                # logging.debug("DASH: %s ,%s", msg_to, data)
                if msg_to == b'COMMAND':
                    logging.debug("DASH RX COMMAND")
                    self._dash_command(json.loads(data))
                    continue
                msg_l = data.split(b'\t')
                if len(msg_l) > 3:
                    control_type = msg_l[2]
                try:
                    device_id = msg_l[1].decode().strip()
                except IndexError:
                    continue
                if control_type == b'ALM':
                    data_topic = f"{self.username}/{device_id}/alarm"
                    control_type = ""
                elif msg_to == b"DASH":
                    data_topic = f"{self.username}/{device_id}/announce"
                else:
                    data_topic = f"{self.username}/{device_id}/data"
                if self._connected and data_topic:
                    logging.debug("DASH TX:\n%s", data.decode().rstrip())
                    self._dash_c.publish(data_topic, data.decode())
            if self._disconnected:
                self.disconnect_timeout = min(self.disconnect_timeout, 900)
                time.sleep(self.disconnect_timeout)
                try:
                    self._dash_c.connect(self.host, self.port)
                except mqtt.socket.gaierror as error:
                    logging.debug("No connection to internet: %s", str(error))
                self.disconnect_timeout = self.disconnect_timeout * 2

        self._dash_c.loop_stop()
        self.tx_zmq_pub.close()
        self.rx_zmq_sub.close()



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from .control import Control, ControlPosition, ControlConfig, _get_color, _get_title_position, _get_label_style
from .enums import Color, LabelStyle, TitlePosition


class LabelConfig(ControlConfig):
    """LabelConfig"""
    def __init__(
        self,
        control_id: str,
        title: str,
        title_position: TitlePosition,
        color: Color,
        style: LabelStyle,
        control_position: ControlPosition
    ) -> None:
        super().__init__(control_id, title, control_position, title_position)
        self.cfg["style"] = style.value
        self.cfg["color"] = str(color.value)

    @classmethod
    def from_dict(cls, cfg_dict: dict):
        """Instantiates LabelConfig from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        LabelConfig
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            _get_color(cfg_dict["color"]),
            _get_label_style(cfg_dict["style"]),
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls


class Label(Control):
    """A Config only control"""

    def __init__(
        self,
        control_id: str,
        title="A label",
        title_position=TitlePosition.BOTTOM,
        color=Color.WHITE,
        style=LabelStyle.BASIC,
        control_position=None,
    ):
        """Label

        Parameters
        ----------
        control_id : str
            An unique control identity string. The control identity string must be a unique string for each control per device
        title : str, optional
            Title of the control, by default "A Label"
        control_position : ControlPosition, optional
            The position of the control on a DeviceView, by default None
        title_position : TitlePosition, optional
            Position of the title when displayed on the iotdashboard app, by default None
        color : Color, optional
            Color of the label, by default Color.WHITE
        style : LabelStyle, optional
            Style of label to be displayed, by default LabelStyle.BASIC
        """
        super().__init__("LBL", control_id)
        control_config = LabelConfig(
            control_id,
            title,
            title_position,
            color,
            style,
            control_position
        )
        self._cfg_columnar.append(control_config)
        self._state_str = ""

    @classmethod
    def from_cfg_dict(cls, cfg_dict: dict):
        """Instatiates a label from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        Knob
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            _get_color(cfg_dict["color"]),
            _get_label_style(cfg_dict["style"]),
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from .control import Control, ControlPosition, ControlConfig, _get_color, _get_title_position, _get_knob_style
from .enums import Color, KnobStyle, TitlePosition


class KnobConfig(ControlConfig):
    """KnobConfig"""
    def __init__(
        self,
        control_id: str,
        title: str,
        title_position: TitlePosition,
        knob_style: KnobStyle,
        dial_min: float,
        dial_max: float,
        red_value: float,
        show_min_max: bool,
        send_only_on_release: bool,
        dial_follows_knob: bool,
        dial_color: Color,
        knob_color: Color,
        control_position=None,
    ) -> None:
        super().__init__(control_id, title, control_position, title_position)
        self.cfg["style"] = str(knob_style.value)
        self.cfg["min"] = dial_min
        self.cfg["max"] = dial_max
        self.cfg["redValue"] = red_value
        self.cfg["showMinMax"] = show_min_max
        self.cfg["sendOnlyOnRelease"] = send_only_on_release
        self.cfg["dialFollowsKnob"] = dial_follows_knob
        self.cfg["dialColor"] = str(dial_color.value)
        self.cfg["knobColor"] = str(knob_color.value)

    @classmethod
    def from_dict(cls, cfg_dict: dict):
        """Instantiates KnobConfig from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        KnobConfig
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            _get_knob_style(cfg_dict["style"]),
            cfg_dict["min"],
            cfg_dict["max"],
            cfg_dict["redValue"],
            cfg_dict["showMinMax"],
            cfg_dict["sendOnlyOnRelease"],
            cfg_dict["dialFollowsKnob"],
            _get_color(cfg_dict["dialColor"]),
            _get_color(cfg_dict["knobColor"]),
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls


class Knob(Control):
    """A Knob control
    """

    def add_config_columnar(self, config: KnobConfig):
        if isinstance(config, KnobConfig):
            config.cfg["min"] = self.dial_min
            config.cfg["max"] = self.dial_max
            config.cfg["redValue"] = self.red_value
            config.cfg["ControlID"] = self.control_id
            self._cfg_columnar.append(config)

    def add_config_full_page(self, config: KnobConfig):
        if isinstance(config, KnobConfig):
            config.cfg["min"] = self.dial_min
            config.cfg["max"] = self.dial_max
            config.cfg["redValue"] = self.red_value
            config.cfg["ControlID"] = self.control_id
            self._cfg_columnar.append(config)

    def __init__(
        self,
        control_id,
        title="A Knob",
        title_position=TitlePosition.BOTTOM,
        knob_style=KnobStyle.NORMAL,
        dial_min=0.0,
        dial_max=100.0,
        red_value=75.0,
        show_min_max=False,
        send_only_on_release=True,
        dial_follows_knob=False,
        dial_color=Color.BLUE,
        knob_color=Color.RED,
        control_position=None
    ):
        """A Knob control is a control with a dial and knob.

        Parameters
        ----------
        control_id : str
            A unique identifier for this control
        title : str, optional
            The title for this control will be displayed on the iotdashboard app, by default "A Knob"
        title_position : TitlePosition, optional
            The position of the title, by default TitlePosition.BOTTOM
        knob_style : KnobStyle, optional
            The Knob style, by default KnobStyle.NORMAL
        dial_min : float, optional
            Minimum dial value, by default 0.0
        dial_max : float, optional
            Maximum dial value, by default 100.0
        red_value : float, optional
            The value where the red starts, by default 75.0
        show_min_max : bool, optional
            Whether to show the min amd max values, by default False
        send_only_on_release : bool, optional
            Have the DashIO app send values either on release or during movement, by default True
        dial_follows_knob : bool, optional
            Have the DashIO app adjust the dial to match the knob value, by default False
        dial_color : Color, optional
            Color of the Dial, by default Color.BLUE
        knob_color : Color, optional
            Color of the Knob, by default Color.RED
        control_position : ControlPosition, optional
            The position of the control on a DeviceView, by default None
        """
        super().__init__("KNOB", control_id)
        self._cfg_columnar.append(
            KnobConfig(
                control_id,
                title,
                title_position,
                knob_style,
                dial_min,
                dial_max,
                red_value,
                show_min_max,
                send_only_on_release,
                dial_follows_knob,
                dial_color,
                knob_color,
                control_position
            )
        )

        self._control_id_dial = f"\t{{device_id}}\tKBDL\t{control_id}\t"
        self._knob_value = 0
        self._knob_dial_value = 0
        self._state_str_knob = self._control_hdr_str + f"{self._knob_value}\n"
        self._state_str_dial = self._control_id_dial + f"{self._knob_dial_value}\n"
        self._knob_dial_state_str = self._state_str_knob + self._state_str_dial

        self._dial_min = dial_min
        self._dial_max = dial_max
        self._red_value = red_value

    @property
    def dial_min(self):
        """Return the minimum dial value"""
        return self._dial_min

    @property
    def dial_max(self):
        """Return the maximum dial value"""
        return self._dial_max

    @property
    def red_value(self):
        """Return the red value for the dial"""
        return self._red_value

    @classmethod
    def from_cfg_dict(cls, cfg_dict: dict):
        """Instatiates Knob from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        Knob
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            _get_knob_style(cfg_dict["style"]),
            cfg_dict["min"],
            cfg_dict["max"],
            cfg_dict["redValue"],
            cfg_dict["showMinMax"],
            cfg_dict["sendOnlyOnRelease"],
            cfg_dict["dialFollowsKnob"],
            _get_color(cfg_dict["dialColor"]),
            _get_color(cfg_dict["knobColor"]),
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls

    def get_state(self):
        return self._knob_dial_state_str

    @property
    def knob_value(self) -> float:
        """knob value

        Returns
        -------
        float
            The value of the knob
        """
        return self._knob_value

    @knob_value.setter
    def knob_value(self, val: float):
        self._knob_value = val
        self._state_str_knob = self._control_hdr_str + f"{val}\n"
        self._message_tx_event(self._state_str_knob)
        self._knob_dial_state_str = self._state_str_knob + self._state_str_dial

    @property
    def knob_dial_value(self) -> float:
        """Knob dial value

        Returns
        -------
        float
            The knob dial value
        """
        return self._knob_dial_value

    @knob_dial_value.setter
    def knob_dial_value(self, val: float):
        self._knob_dial_value = val
        self._state_str_dial = self._control_id_dial + f"{val}\n"
        self._message_tx_event(self._state_str_dial)
        self._knob_dial_state_str = self._state_str_knob + self._state_str_dial



import json
import unittest

from dashio import Button, ButtonState, Icon


class TestButton(unittest.TestCase):
    def _get_cfg_dict(self, cfg_str):
        json_str = cfg_str.rpartition('\t')[2]
        return json.loads(json_str)

    def test_button_state_on(self):
        test_control = Button("BUTTONID")
        test_control.btn_state = ButtonState.ON
        self.assertEqual(test_control.get_state(), '\t{device_id}\tBTTN\tBUTTONID\tON\n', "Should be 'ON'")

    def test_button_state_off(self):
        test_control = Button("BUTTONID")
        test_control.btn_state = ButtonState.OFF
        self.assertEqual(test_control.get_state(), '\t{device_id}\tBTTN\tBUTTONID\tOFF\n', "Should be 'OFF'")
    """
    def test_button_state_flash(self):
        test_control = Button("BUTTONID")
        test_control.btn_state = ButtonState.FLASH
        self.assertEqual(test_control.get_state(), '\t{device_id}\tBTTN\tBUTTONID\tFLASH\n', "Should be 'FLASH'")
    """
    def test_button_set_icon(self):
        test_control = Button("BUTTONID")
        test_control.icon_name = Icon.UP
        self.assertEqual(test_control.get_state(), '\t{device_id}\tBTTN\tBUTTONID\tOFF\tUp\n', "Icon Should be UP")

    def test_button_set_text(self):
        test_control = Button("BUTTONID")
        test_control.text = "HELLO"
        self.assertEqual(test_control.get_state(), '\t{device_id}\tBTTN\tBUTTONID\tOFF\tNone\tHELLO\n', "text Should be Hello")

    def test_button_text_icon(self):
        test_control = Button("BUTTONID")
        test_control.send_button(ButtonState.ON, Icon.PAD, "HELLO")
        self.assertEqual(test_control.get_state(), '\t{device_id}\tBTTN\tBUTTONID\tON\tPad\tHELLO\n', "icon Should be Pad")


if __name__ == '__main__':
    unittest.main()



import json
import unittest
from datetime import datetime

import dateutil
from dashio import EventData, EventLog
from dashio.iotcontrol.enums import Color


class TestEventLog(unittest.TestCase):
    def _get_cfg_dict(self, cfg_list: list):
        json_str = cfg_list[0].rpartition('\t')[2]
        return json.loads(json_str)

    def test_event_data(self):
        test_data = EventData("HEADER")
        test_str_list = str(test_data).split('\t')
        self.assertIsInstance(dateutil.parser.isoparse(test_str_list[0]), datetime, "Should be datetime")
        self.assertEqual(Color(int(test_str_list[1])), Color.WHITE, "Color whould be WHITE")

    def test_event_log_control_type(self):
        test_control = EventLog("EVENTLOGID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[2], 'LOG', "control type should be LOG")

    def test_event_log_control_id(self):
        test_control = EventLog("EVENTLOGID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[3], 'EVENTLOGID', "control_id type should be EVENTLOGID")


if __name__ == '__main__':
    unittest.main()



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import json
import logging
import threading

import zmq

from ..constants import CONNECTION_PUB_URL, MEMORY_REQ_URL, TASK_PULL


def num(s_num: str):
    try:
        return int(s_num)
    except ValueError:
        return float(s_num)


def _read_control_task(data, task):
    logging.debug("READ_CONTROL: %s", data)
    return data[+3]


def _if_task(data, task):
    logging.debug("IF: %s", data)
    return ""


def _endif_task(data, task):
    logging.debug("ENDIF: %s", data)
    return ''


def _send_alarm_task(data, task):
    logging.debug("SEND_ALARM: %s", data)
    return ''


def _write_control_task(data, task):
    logging.debug("WRITE_CONTROL: %s", data)
    return ""


MESSAGE_FORMAT_INPUTS = {
    "AVD": ["url"],
    "BTTN": ["button_state", "icon_name", "text"],
    "TEXT": ["text"],
    "GRPH": ["line_id", "line_data"],
    "DIAL": ["data"],
    "CLR": ["rgb"],
    "TGRPH": ["time_stamp", "data"],
    "KNOB": ["data"],
    "KBDL": ["data"],
    "SLCTR": ["url"],
    "SLDR": ["url"],
    "DIR": ["url"],
    "LOG": ["url"],
    "LBL": ["url"],
}

MESSAGE_FORMAT_OUTPUTS = {
    "AVD": "\t{device_id}\tAVD\t{control_id}\t{url}",
    "BTTN": "\t{device_id}\tBTTN\t{control_id}\t{url}",
    "TEXT": "\t{device_id}\tTEXT\t{control_id}\t{url}",
    "GRPH": "\t{device_id}\tGRPH\t{control_id}\t{url}",
    "DIAL": "\t{device_id}\tDIAL\t{control_id}\t{url}",
    "CLR": "\t{device_id}\tCLR\t{control_id}\t{url}",
    "TGRPH": "\t{device_id}\tTGRPH\t{control_id}\t{url}",
    "KNOB": "\t{device_id}\tKNOB\t{control_id}\t{url}",
    "KBDL": "\t{device_id}\tKBDL\t{control_id}\t{url}",
    "SLCTR": "\t{device_id}\tSLCTR\t{control_id}\t{url}",
    "SLDR": "\t{device_id}\tSLDR\t{control_id}\t{url}",
    "DIR": "\t{device_id}\tDIR\t{control_id}\t{url}",
    "LOG": "\t{device_id}\tLOG\t{control_id}\t{url}",
    "LBL": "\t{device_id}\tLBL\t{control_id}\t{url}",
}


class TaskService(threading.Thread):
    """Task Class"""

    def send_message(self, out_message=""):
        """Send the message"""
        self.task_sender.send_multipart([b"ALL", out_message.encode()])

    def close(self):
        """Close the thread"""
        self.running = False

    def _task_store_mem(self, mem_loc, data):
        self.task_memory[mem_loc] = data

    def _task_get_mem(self, mem_loc):
        return self.task_memory.get(mem_loc, "")

    def _send_alarm(self, alarm_id: str, header: str, body: str):
        msg = f"\t{self.device_id}\tALM\t{alarm_id}\t{header}\t{body}\n"
        self.send_message(msg)

    def _read_control_action(self, action, msg: bytearray):
        logging.debug("READ_CONTROL: %s", msg.decode())

    def _send_alarm_action(self, action, msg):
        logging.debug("SEND_ALARM: %s", msg)
        self._send_alarm(action['alarmID'], action['title'], action['body'])

    def _write_control_action(self, action, msg):
        logging.debug("WRITE_CONTROL: %s", msg.decode())

    def _read_mem_action(self, action, msg):
        logging.debug("READ_MEM memType: %s", action["memType"])
        if action["memType"] == "Local":
            pass
        elif action["memType"] == "Task":
            reply = self._task_get_mem(action['memoryID'])
            logging.debug("Task read mem: %s", reply)
        elif action["memType"] == "Global":
            self.device_mem_socket.send_multipart([b'GET', action['memoryID'].encode(), b'0'])
            reply = self.device_mem_socket.recv_multipart()
        else:
            logging.debug("READ_MEM unknown memType: %s", action["memType"])

    def _write_mem_action(self, action, msg):
        logging.debug("WRITE_MEM memType: %s", action["memType"])
        if action["memType"] == "Local":
            pass
        elif action["memType"] == "Task":
            self._task_store_mem(action['memoryID'], action['thing'])
        elif action["memType"] == "Global":
            self.device_mem_socket.send_multipart([b'SET', action['memoryID'].encode(), action['thing'].encode()])
            reply = self.device_mem_socket.recv_multipart()
            logging.debug("Task write mem: %s", reply)
        else:
            logging.debug("WRITE_MEM unknown memType: %s", action["memType"])

    def _connect_device_id(self, device_id):
        msg_dict = {
            'msgType': 'connect',
            'deviceID': device_id
        }
        logging.debug("AS CONNECT: %s", device_id)
        self.task_sender.send_multipart([b"COMMAND", json.dumps(msg_dict).encode()])

    def __init__(self, device_id: str, action_station_id: str, task_config_dict: dict, context: zmq.Context) -> None:
        threading.Thread.__init__(self, daemon=True)
        self.context = context
        self.running = True
        self.timer_type = None
        self.device_id = device_id
        self.control_type = "TASK"
        self.task_memory = {}

        self.action_function_dict = {
            "READ_CONTROL": self._read_control_action,
            "SEND_ALARM": self._send_alarm_action,
            "WRITE_CONTROL": self._write_control_action,
            "READ_MEM": self._read_mem_action,
            "WRITE_MEM": self._write_mem_action,
        }

        self.task_id = task_config_dict['uuid']
        self.actions = task_config_dict['actions']
        self.name = task_config_dict['name']
        self.sub_msg = None
        rx_device_id = ""
        if len(self.actions) == 0:
            return
        try:
            rx_device_id = self.actions[0]["deviceID"]
            rx_control_type = self.actions[0]["controlType"]
            rx_control_id = self.actions[0]["controlID"]
        except KeyError:
            return
        self.sub_msg = f"\t{rx_device_id}\t{rx_control_type}\t{rx_control_id}"
        self.push_url = TASK_PULL.format(id=action_station_id)
        self.sub_url = CONNECTION_PUB_URL.format(id=action_station_id)

        self.device_mem_socket = self.context.socket(zmq.REQ)
        self.device_mem_socket.connect(MEMORY_REQ_URL.format(id=self.device_id))

        self.task_sender = self.context.socket(zmq.PUSH)
        self.task_sender.connect(self.push_url)

        if rx_device_id != device_id:
            self._connect_device_id(rx_device_id)
        self.start()

    # Do a simple test case to check messaging.
    def _do_actions(self, msg):
        for action in self.actions:
            self.action_function_dict[action['objectType']](action, msg)

    def run(self):
        task_receiver = self.context.socket(zmq.SUB)
        task_receiver.connect(self.sub_url)
        task_receiver.setsockopt_string(zmq.SUBSCRIBE, self.sub_msg)

        poller = zmq.Poller()
        poller.register(task_receiver, zmq.POLLIN)

        while self.running:
            try:
                socks = dict(poller.poll(15))
            except zmq.error.ContextTerminated:
                break
            if task_receiver in socks:
                message, _ = task_receiver.recv_multipart()
                if message:
                    logging.debug("TASK: %s\t%s RX:%s", self.name, self.task_id, message.decode())
                    self._do_actions(message)
        self.task_sender.close()
        task_receiver.close()
        self.device_mem_socket.close()



"""Modbus"""
import glob
import logging
import sys
import threading

import serial
import zmq

from ..constants import TASK_PULL, CONNECTION_PUB_URL
from .action_station_service_config import (
    ActionServiceCFG,
    IntParameterSpec,
    ListParameterSpec,
    SelectorParameterSpec
)


def serial_ports():
    """ Lists serial port names

        :raises EnvironmentError:
            On unsupported or unknown platforms
        :returns:
            A list of the serial ports available on the system
    """
    if sys.platform.startswith('win'):
        ports = ['COM%s' % (i + 1) for i in range(256)]
    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this excludes your current terminal "/dev/tty"
        ports = glob.glob('/dev/tty[A-Za-z]*')
    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')
    else:
        raise EnvironmentError('Unsupported platform')

    result = []
    for port in ports:
        try:
            s = serial.Serial(port)
            s.close()
            result.append(port)
        except (OSError, serial.SerialException):
            pass
    return result


def make_modbus_config(num_tests):
    """Make a timer config"""
    s_ports = serial_ports()
    default = ""
    if len(s_ports) > 0:
        default = s_ports[0]
    provisioning_list = [
        SelectorParameterSpec("Serial Port", s_ports, default),
        SelectorParameterSpec("Baud", ["1200", "2400", "4800", "9600", "19200", "38400", "57600", "115200"], "9600"),
        SelectorParameterSpec("Modbus Type", ["RTU", "ASCII"], "RTU"),
        ListParameterSpec(
            "Read Registers",
            "Create a list of registers to read data from.",
            [
                IntParameterSpec(
                    "Read Register Base Address",
                    0,
                    65535,
                    "",
                    0
                ),
                SelectorParameterSpec(
                    "Register Type", [
                        "char",
                        "int_8",
                        "uint_8",
                        "int_16",
                        "uint_16",
                        "int_32",
                        "uint_32",
                        "int_64",
                        "uint_64"
                    ],
                    "uint_16",
                ),
                IntParameterSpec(
                    "Number of register to read from base address (Optional).",
                    0,
                    65535,
                    "",
                    2
                )
            ]
        ),
        ListParameterSpec(
            "Read a Bit",
            "Create a list of registers to read data from.",
            [
                IntParameterSpec(
                    "Read Register Base Address",
                    0,
                    65535,
                    "",
                    0
                ),
                IntParameterSpec(
                    "Bit Mask",
                    0,
                    65535,
                    "",
                    0
                )
            ]
        )

    ]
    parameter_list_in = []
    # parameter_list_out = []
    timer_cfg = ActionServiceCFG(
        "MDBS",
        "Modbus Test",
        "Modbus - Test provisioning of list",
        "TST$",
        num_tests,
        True,
        True,
        provisioning_list,
        parameter_list_in
        #  parameter_list_out
    )
    return timer_cfg.__json__()


class ModbusService(threading.Thread):
    """Action Station Template Class"""

    def send_message(self, out_message=""):
        """Send the message"""
        self.task_sender.send_multipart([b"ALL", out_message.encode('utf-8')])

    def close(self):
        """Close the thread"""
        self.running = False

    def __init__(self, device_id: str, action_station_id: str, control_config_dict: dict, context: zmq.Context) -> None:
        threading.Thread.__init__(self, daemon=True)

        self.context = context
        self.running = True

        self.control_id = control_config_dict['controlID']
        self.name = control_config_dict['name']
        self.control_type = control_config_dict['objectType']
        # provision_list = control_config_dict['provisioning']

        self.push_url = TASK_PULL.format(id=action_station_id)

        self.sub_url = CONNECTION_PUB_URL.format(id=action_station_id)

        self.task_sender = self.context.socket(zmq.PUSH)
        self.task_sender.connect(self.push_url)

        self.control_msg = f"\t{device_id}\t{self.control_type}\t{self.control_id}"
        logging.debug("Init Class: %s, %s", self.control_type, self.name)

        self.start()

    def run(self):
        receiver = self.context.socket(zmq.SUB)
        receiver.connect(self.sub_url)
        receiver.setsockopt_string(zmq.SUBSCRIBE, self.control_msg)
        poller = zmq.Poller()
        poller.register(receiver, zmq.POLLIN)

        while self.running:
            try:
                socks = dict(poller.poll(15))
            except zmq.error.ContextTerminated:
                break
            if receiver in socks:
                message, _ = receiver.recv()
                if message:
                    logging.debug("%s\t%s RX:\n%s", self.control_type, self.control_id, message.decode())

        self.task_sender.close()
        receiver.close()



import json
import unittest

from dashio import Control


class TestControl(unittest.TestCase):
    def _get_cfg_dict(self, cfg_str):
        json_str = cfg_str.rpartition('\t')[2]
        return json.loads(json_str)

    def test_control_id(self):
        test_control = Control("CONTROLTYPE", "CONTROLID")
        self.assertEqual(test_control._control_hdr_str, '\t{device_id}\tCONTROLTYPE\tCONTROLID\t', "ControlID Should be CONTROLID")


if __name__ == '__main__':
    unittest.main()



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from .control import Control, ControlPosition, ControlConfig, _get_title_position, _get_color, _get_bar_style
from .enums import Color, SliderBarStyle, TitlePosition


class SliderConfig(ControlConfig):
    """SliderConfig"""
    def __init__(
        self,
        control_id: str,
        title: str,
        title_position: TitlePosition,
        bar_min: float,
        bar_max: float,
        red_value: float,
        show_min_max: bool,
        slider_enabled: bool,
        send_only_on_release: bool,
        bar_follows_slider: bool,
        bar_color: Color,
        knob_color: Color,
        bar_style: SliderBarStyle,
        control_position: ControlPosition,
    ) -> None:
        super().__init__(control_id, title, control_position, title_position)
        self.cfg["redValue"] = red_value
        self.cfg["min"] = bar_min
        self.cfg["max"] = bar_max
        self.cfg["showMinMax"] = show_min_max
        self.cfg["sliderEnabled"] = slider_enabled
        self.cfg["sendOnlyOnRelease"] = send_only_on_release
        self.cfg["barFollowsSlider"] = bar_follows_slider
        self.cfg["barColor"] = str(bar_color.value)
        self.cfg["knobColor"] = str(knob_color.value)
        self.cfg["barStyle"] = bar_style.value

    @classmethod
    def from_dict(cls, cfg_dict: dict):
        """Instantiates SliderConfig from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        SliderConfig
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            cfg_dict["min"],
            cfg_dict["max"],
            cfg_dict["redValue"],
            cfg_dict["showMinMax"],
            cfg_dict["sliderEnabled"],
            cfg_dict["sendOnlyOnRelease"],
            cfg_dict["barFollowsSlider"],
            _get_color(cfg_dict["barColor"]),
            _get_color(cfg_dict["knobColor"]),
            _get_bar_style(cfg_dict["barStyle"]),
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls


class Slider(Control):
    """Single slider bar control
    """

    def add_config_columnar(self, config: SliderConfig):
        if isinstance(config, SliderConfig):
            config.cfg["min"] = self.bar_min
            config.cfg["max"] = self.bar_max
            config.cfg["redValue"] = self.red_value
            config.cfg["ControlID"] = self.control_id
            self._cfg_columnar.append(config)

    def add_config_full_page(self, config: SliderConfig):
        if isinstance(config, SliderConfig):
            config.cfg["min"] = self.bar_min
            config.cfg["max"] = self.bar_max
            config.cfg["redValue"] = self.red_value
            config.cfg["ControlID"] = self.control_id
            self._cfg_columnar.append(config)

    def __init__(
        self,
        control_id: str,
        title="A Single Slider",
        title_position=TitlePosition.BOTTOM,
        bar_min=0.0,
        bar_max=1000.0,
        red_value=750,
        show_min_max=False,
        slider_enabled=True,
        send_only_on_release=True,
        bar_follows_slider=False,
        bar_color=Color.BLUE,
        knob_color=Color.RED,
        bar_style=SliderBarStyle.SEG,
        control_position=None,
    ):
        """[summary]

        Parameters
        ----------
        control_id : str
            An unique control identity string. The control identity string must be a unique string for each control per device
        title : str, optional
            Title of the control, by default "A Single SLider"
        control_position : ControlPosition, optional
            The position of the control on a DeviceView, by default None
        title_position : TitlePosition, optional
            Position of the title when displayed on the iotdashboard app, by default None
        bar_min : float, optional
            min bar value, by default 0.0
        bar_max : float, optional
            max bar vale, by default 1000.0
        red_value : int, optional
            red value, by default 750
        show_min_max : bool, optional
            show min max, by default False
        slider_enabled : bool, optional
            enable slider, by default True
        send_only_on_release : bool, optional
            send only on release, by default True
        bar_follows_slider : bool, optional
            bar follows slider, by default False
        bar_color : Color, optional
            bar color, by default Color.BLUE
        knob_color : Color, optional
            knob color, by default Color.RED
        bar_style : SliderBarStyle, optional
            bar style, by default SliderBarStyle.SEGMENTED
        """
        super().__init__("SLDR", control_id)
        self._cfg_columnar.append(
            SliderConfig(
                control_id,
                title,
                title_position,
                bar_min,
                bar_max,
                red_value,
                show_min_max,
                slider_enabled,
                send_only_on_release,
                bar_follows_slider,
                bar_color,
                knob_color,
                bar_style,
                control_position
            )
        )

        self._control_id_bar = f"\t{{device_id}}\tBAR\t{control_id}\t"

        self._bar1_value = 0.0
        self._bar2_value = None
        self._slider_value = 0.0

        self._slider_state_str = self._control_hdr_str + f"{self._slider_value}\n"
        self._bar_state_str = self._control_id_bar + f"{self._bar1_value}\n"
        self._bar_slider_state_str = self._slider_state_str + self._bar_state_str

        self._bar_min = bar_min
        self._bar_max = bar_max
        self._red_value = red_value

    @property
    def bar_min(self):
        """Returns the minimum bar value"""
        return self._bar_min

    @property
    def bar_max(self):
        """Returns the maximum bar value"""
        return self._bar_max

    @property
    def red_value(self):
        """Returns the red value"""
        return self._red_value

    @classmethod
    def from_cfg_dict(cls, cfg_dict: dict):
        """Instatiates Slider from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        Slider
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            cfg_dict["min"],
            cfg_dict["max"],
            cfg_dict["redValue"],
            cfg_dict["showMinMax"],
            cfg_dict["sliderEnabled"],
            cfg_dict["sendOnlyOnRelease"],
            cfg_dict["barFollowsSlider"],
            _get_color(cfg_dict["barColor"]),
            _get_color(cfg_dict["knobColor"]),
            _get_bar_style(cfg_dict["barStyle"]),
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls

    def get_state(self):
        return self._bar_slider_state_str

    @property
    def bar1_value(self) -> float:
        """bar 1 value

        Returns
        -------
        float
            the bar value
        """
        return self._bar1_value

    @bar1_value.setter
    def bar1_value(self, val: float):
        self._bar1_value = val

        if self._bar2_value is None:
            self._bar_state_str = self._control_id_bar + f"{self._bar1_value}\n"
        else:
            self._bar_state_str = self._control_id_bar + "{:.2f}\t{:.2f}\n".format(self._bar1_value, self._bar2_value)
        self._message_tx_event(self._bar_state_str)
        self._bar_slider_state_str = self._slider_state_str + self._bar_state_str

    @property
    def bar2_value(self) -> float:
        """bar2 value

        Returns
        -------
        float
            The value of bar2
        """
        return self._bar2_value

    @bar2_value.setter
    def bar2_value(self, val: float):
        self._bar2_value = val
        if self._bar2_value is None:
            self._bar_state_str = self._control_id_bar + f"{self._bar1_value}\n"
        else:
            self._bar_state_str = self._control_id_bar + "{:.2f}\t{:.2f}\n".format(self._bar1_value, self._bar2_value)
        self._message_tx_event(self._bar_state_str)
        self._bar_slider_state_str = self._slider_state_str + self._bar_state_str

    @property
    def slider_value(self) -> float:
        """the slider value

        Returns
        -------
        float
            Slider value
        """
        return self._slider_value

    @slider_value.setter
    def slider_value(self, val: float):
        self._slider_value = val
        self._slider_state_str = self._control_hdr_str + f"{self._slider_value}\n"
        self._message_tx_event(self._slider_state_str)
        self._bar_slider_state_str = self._slider_state_str + self._bar_state_str



import json
import unittest

from dashio import TextBox
from dashio.iotcontrol.enums import (
    Keyboard,
    Precision,
    TextAlignment,
    TextFormat
)


def _get_cfg_dict(cfg_list: list):
    json_str = cfg_list[0].rpartition('\t')[2]
    return json.loads(json_str)


class TestTextBox(unittest.TestCase):

    def test_text_box_control_type(self):
        test_control = TextBox("TEXTBOXID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[2], 'TEXT', "control type should be TEXT")

    def test_text_box_control_id(self):
        test_control = TextBox("TEXTBOXID")
        test_str_list = test_control._control_hdr_str.split('\t')
        self.assertEqual(test_str_list[3], 'TEXTBOXID', "control type should be TEXTBOXID")

    def test_text_box_cfg_text_align(self):
        test_control = TextBox("TEXTBOXID", text_align=TextAlignment.CENTER)
        cfg_dict = _get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(TextAlignment(cfg_dict['textAlign']), TextAlignment.CENTER, "CFG textAlign should be CENTER")

    def test_text_box_cfg_text_format(self):
        test_control = TextBox("TEXTBOXID", text_format=TextFormat.NUMBER)
        cfg_dict = _get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(TextFormat(cfg_dict['format']), TextFormat.NUMBER, "CFG format should be CENTER")

    def test_text_box_cfg_units(self):
        test_control = TextBox("TEXTBOXID", units='Amps')
        cfg_dict = _get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['units'], 'Amps', "CFG units should be 'Amps'")

    def test_text_box_cfg_keyboard_type(self):
        test_control = TextBox("TEXTBOXID", keyboard_type=Keyboard.HEX)
        cfg_dict = _get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(Keyboard(cfg_dict['kbdType']), Keyboard.HEX, "CFG kbdType should be HEX")

    def test_text_box_cfg_precision(self):
        test_control = TextBox("TEXTBOXID", precision=Precision.SIX)
        cfg_dict = _get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(Precision(cfg_dict['precision']), Precision.SIX, "CFG precision should be SIX")

    def test_text_box_cfg_close_keyboard_on_send(self):
        test_control = TextBox("TEXTBOXID", close_keyboard_on_send=True)
        cfg_dict = _get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['closeKbdOnSend'], True, "CFG closeKbdOnSend should be True")


if __name__ == '__main__':
    unittest.main()



import json
import unittest

from dashio import DashConnection


class TestDashConnection(unittest.TestCase):
    def _get_cfg_dict(self, cfg_str):
        json_str = cfg_str.rpartition('\t')[2]
        return json.loads(json_str)

    def test_dash_connection_username(self):
        test_connection = DashConnection("USERNAME", "PASSWORD")
        self.assertEqual(test_connection.username, 'USERNAME', "username type should be USERNAME")


if __name__ == '__main__':
    unittest.main()



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from .control import Control, ControlPosition, ControlConfig, _get_color, _get_title_position, _get_dial_number_position, _get_dial_style, _get_precision
from .enums import (Color, DialNumberPosition, DialStyle, Precision,
                    TitlePosition)


class DialConfig(ControlConfig):
    """DialConfig"""
    def __init__(
        self,
        control_id: str,
        title: str,
        title_position: TitlePosition,
        dial_min: float,
        dial_max: float,
        red_value: float,
        dial_fill_color: Color,
        pointer_color: Color,
        number_position: DialNumberPosition,
        show_min_max: bool,
        style: DialStyle,
        precision: Precision,
        units: str,
        control_position: ControlPosition
    ) -> None:
        super().__init__(control_id, title, control_position, title_position)
        self.cfg["min"] = dial_min
        self.cfg["max"] = dial_max
        self.cfg["redValue"] = red_value
        self.cfg["dialFillColor"] = str(dial_fill_color.value)
        self.cfg["pointerColor"] = str(pointer_color.value)
        self.cfg["numberPosition"] = number_position.value
        self.cfg["showMinMax"] = show_min_max
        self.cfg["style"] = style.value
        self.cfg["precision"] = precision.value
        self.cfg["units"] = units

    @classmethod
    def from_dict(cls, cfg_dict: dict):
        """Instantiates DialConfig from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        DialConfig
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            cfg_dict["min"],
            cfg_dict["max"],
            cfg_dict["redValue"],
            _get_color(cfg_dict["dialFillColor"]),
            _get_color(cfg_dict["pointerColor"]),
            _get_dial_number_position(cfg_dict["numberPosition"]),
            cfg_dict["showMinMax"],
            _get_dial_style(cfg_dict["style"]),
            _get_precision(cfg_dict["precision"]),
            cfg_dict["units"],
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls


class Dial(Control):
    """Dial Control"""

    def add_config_columnar(self, config: DialConfig):
        if isinstance(config, DialConfig):
            config.cfg["min"] = self.dial_min
            config.cfg["max"] = self.dial_max
            config.cfg["redValue"] = self.red_value
            config.cfg["ControlID"] = self.control_id
            self._cfg_columnar.append(config)

    def add_config_full_page(self, config: DialConfig):
        if isinstance(config, DialConfig):
            config.cfg["min"] = self.dial_min
            config.cfg["max"] = self.dial_max
            config.cfg["redValue"] = self.red_value
            config.cfg["ControlID"] = self.control_id
            self._cfg_columnar.append(config)

    def __init__(
        self,
        control_id: str,
        title="A Dial",
        title_position=TitlePosition.BOTTOM,
        dial_min=0.0,
        dial_max=100.0,
        red_value=75.0,
        dial_fill_color=Color.RED,
        pointer_color=Color.BLUE,
        number_position=DialNumberPosition.LEFT,
        show_min_max=False,
        style=DialStyle.PIE,
        precision=Precision.OFF,
        units="",
        control_position=None
    ):
        """Dial

        Parameters
        ----------
        control_id : str
            An unique control identity string. The control identity string must be a unique string for each control per device
        title : str, optional
            Title of the control, by default None
        control_position : ControlPosition, optional
            The position of the control on a DeviceView, by default None
        title_position : TitlePosition, optional
            Position of the title when displayed on the iotdashboard app, by default None
        dial_min : float, optional
            Minimum dial position, by default 0.0
        dial_max : float, optional
            Maximum dial position, by default 100.0
        red_value : float, optional
            Position for red part of dial, by default 75.0
        dial_fill_color : Color, optional
            Dial fill color, by default Color.RED
        pointer_color : Color, optional
            Pointer color, by default Color.BLUE
        number_position : DialNumberPosition, optional
            Position of the number on the dial, by default DialNumberPosition.LEFT
        show_min_max : bool, optional
            True to show min max, by default False
        style : DialStyle, optional
            Dial style, by default DialStyle.PIE
        precision : Precision, optional
            Precision of the displayed number, by default Precision.OFF
        units : str, optional
            Units of the dial position, by default ""
        """
        super().__init__("DIAL", control_id)
        self._cfg_columnar.append(
            DialConfig(
                control_id,
                title,
                title_position,
                dial_min,
                dial_max,
                red_value,
                dial_fill_color,
                pointer_color,
                number_position,
                show_min_max,
                style,
                precision,
                units,
                control_position
            )
        )
        self._dial_value = 0
        self._dial_min = dial_min
        self._dial_max = dial_max
        self._red_value = red_value

    @property
    def dial_min(self):
        """Return the minimum dial value"""
        return self._dial_min

    @property
    def dial_max(self):
        """Return the maximum dial value"""
        return self._dial_max

    @property
    def red_value(self):
        """Return the red value for the dial"""
        return self._red_value

    @classmethod
    def from_cfg_dict(cls, cfg_dict: dict):
        """Instatiates Dial from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        Dial
        """

        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            cfg_dict["min"],
            cfg_dict["max"],
            cfg_dict["redValue"],
            _get_color(cfg_dict["dialFillColor"]),
            _get_color(cfg_dict["pointerColor"]),
            _get_dial_number_position(cfg_dict["numberPosition"]),
            cfg_dict["showMinMax"],
            _get_dial_style(cfg_dict["style"]),
            _get_precision(cfg_dict["precision"]),
            cfg_dict["units"],
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls

    def get_state(self):
        return self._control_hdr_str + f"{self._dial_value}\n"

    @property
    def dial_value(self) -> float:
        """Dial value

        Returns
        -------
        float
            The position of the dial
        """
        return self._dial_value

    @dial_value.setter
    def dial_value(self, val: float):
        self._dial_value = val
        self.state_str = self._control_hdr_str + f"{val}\n"



import shortuuid


class BaseParameter:
    """Base class for Parameter"""
    def __init__(self, param_type: str, name: str):
        self._param = {
            'objectType': param_type,
            'name': name,
            'uuid': shortuuid.uuid()
        }

    def __json__(self, **options):
        return self._param


class ListParameterSpec(BaseParameter):
    """List Parameter"""
    def __init__(self, name: str, text: str, param_list: list):
        super().__init__("LIST_PARAM", name)
        self._param['text'] = text
        self._param["list"] = param_list


class StringParameterSpec(BaseParameter):
    """String Parameter"""
    def __init__(self, name: str, existing_value: int):
        super().__init__("STRING_PARAM", name)
        self._param['value'] = existing_value


class FloatParameterSpec(BaseParameter):
    """Float Parameter"""
    def __init__(self, name: str, param_min: float, param_max: float, units: str, existing_value: float):
        super().__init__("FLOAT_PARAM", name)
        self._param['min'] = param_min
        self._param['max'] = param_max
        self._param['units'] = units
        self._param['value'] = existing_value


class IntParameterSpec(BaseParameter):
    """Int Parameter"""
    def __init__(self, name: str, param_min: int, param_max: int, units: str, existing_value: int):
        super().__init__("INT_PARAM", name)
        self._param['min'] = param_min
        self._param['max'] = param_max
        self._param['units'] = units
        self._param['value'] = existing_value


class BoolParameterSpec(BaseParameter):
    """Bool Parameter"""
    def __init__(self, name: str, existing_value: bool):
        super().__init__("BOOL_PARAM", name)
        self._param['value'] = existing_value


class SelectorParameterSpec(BaseParameter):
    """Selection Parameter"""
    def __init__(self, name: str, selection: list, existing_value: str):
        super().__init__("SELECTION_PARAM", name)
        self._param['selection'] = selection
        self._param['value'] = existing_value


class SliderParameterSpec(BaseParameter):
    """Slider Parameter"""
    def __init__(self, name: str, param_min: float, param_max: float, step: float, existing_value: str):
        super().__init__("SLIDER_PARAM", name)
        self._param['min'] = param_min
        self._param['max'] = param_max
        self._param['step'] = step
        self._param['value'] = existing_value


class ActionServiceCFG:
    """Action Control CFG"""
    def __init__(
        self,
        control_type: str,
        name: str,
        text: str,
        control_id: str,
        num_avail: int,
        is_trigger: bool,
        is_io: bool,
        provisioning_list: list,
        param_in_list: list,
        revision=1
    ) -> None:
        self.uuid = shortuuid.uuid()
        self._config_def = {
            'objectType': "CONFIG",
            'objectName': control_type,
            'revision': revision,
            'uuid': self.uuid,
            'name': name,
            'text': text,
            'controlID': control_id,
            'numAvail': num_avail,
            'isTrigger': is_trigger,
            'isIO': is_io,
            'provisioning': provisioning_list,
            'parameters': param_in_list,
            #  'parametersOut': param_out_list
        }

    def __json__(self, **options):
        return self._config_def



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


class RingBuffer:
    """ class that implements a not-yet-full buffer"""
    def __init__(self, size_max):
        self.max = size_max
        self.data = []
        self._full = False

    def append(self, val):
        """ append an element at the end of the buffer."""
        if len(self.data) == self.max:
            self.cur = 0
            # Permanently change self's class from non-full to full
            self._full = True
        if self._full:
            # Append an element overwriting the oldest one.
            # pylint: disable=access-member-before-definition
            self.data[self.cur] = val
            self.cur = (self.cur + 1) % self.max
        else:
            self.data.append(val)

    def get_latest(self):
        """Get the last item in the buffer
        """
        if self._full:
            # Get the last item in the buffer
            indx = self.cur - 1
            if indx < 0:
                indx = self.max - 1
            return self.data[indx]
        return self.data[-1]

    def empty(self) -> bool:
        """
        Returns if empty
        """
        if not self.data:
            return True
        return False

    def get(self):
        """ Return a list of elements from the oldest to the newest."""
        if self._full:
            # return list of elements in correct order
            return self.data[self.cur:] + self.data[:self.cur]
        return self.data



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from ..constants import BAD_CHARS
from .control import Control, ControlPosition, ControlConfig, _get_title_position, _get_color_picker_style
from .enums import (ColorPickerStyle, TitlePosition)


class ColorPickerConfig(ControlConfig):
    """ColorPickerConfig"""
    def __init__(
        self,
        control_id: str,
        title: str,
        title_position: TitlePosition,
        style: ColorPickerStyle,
        send_only_on_release: bool,
        control_position: ControlPosition
    ) -> None:
        super().__init__(control_id, title, control_position, title_position)
        self.cfg["style"] = style.value
        self.cfg["sendOnlyOnRelease"] = send_only_on_release

    @classmethod
    def from_dict(cls, cfg_dict: dict):
        """Instantiates ColorPickerConfig from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        ColorPickerConfig
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            _get_color_picker_style(cfg_dict["pickerStyle"]),
            cfg_dict["sendOnlyOnRelease"],
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls


class ColorPicker(Control):
    """Color Picker Control
    """

    def __init__(
        self,
        control_id: str,
        title="A Color Picker",
        title_position=TitlePosition.BOTTOM,
        style=ColorPickerStyle.WHEEL,
        send_only_on_release=True,
        control_position=None,
    ):
        """ColorPicker

        Parameters
        ----------
        control_id : str
            An unique control identity string. The control identity string must be a unique string for each control per device
        title : str, optional
            Title of the control, by default None
        control_position : ControlPosition, optional
            The position of the control on a DeviceView, by default None
        title_position : TitlePosition, optional
            Position of the title when displayed on the iotdashboard app, by default None
        picker_style: ColorPickerStyle, optional
            The style of color picker to use.
        send_only_on_release: Boolean
            send only on release, by default True

        """
        super().__init__("CLR", control_id)
        self._cfg_columnar.append(
            ColorPickerConfig(
                control_id,
                title,
                title_position,
                style,
                send_only_on_release,
                control_position
            )
        )
        self._color_value = "#4F5GA2"

    @classmethod
    def from_cfg_dict(cls, cfg_dict: dict):
        """Instatiates ColorPicker from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        Button
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            _get_title_position(cfg_dict["titlePosition"]),
            _get_color_picker_style(cfg_dict["pickerStyle"]),
            cfg_dict["sendOnlyOnRelease"],
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls

    def get_state(self):
        return self._control_hdr_str + f"{self._color_value}\n"

    def send_color_rgb(self, red: int, green: int, blue: int):
        """Send color as #rrggbb for the given color values."""
        self._color_value = f"#{red:02x}{green:02x}{blue:02x}"
        self.state_str = self._control_hdr_str + f"{self._color_value}\n"

    def color_to_rgb(self, color_value: str) -> tuple:
        """Return (red, green, blue) for the color_value."""
        clr = color_value.lstrip('#')
        len_v = len(clr)
        return tuple(int(clr[i:i + len_v // 3], 16) for i in range(0, len_v, len_v // 3))

    @property
    def color_value(self) -> str:
        """Color value

        Returns
        -------
        str
            The color value as #rrggbb
        """
        return self._color_value

    @color_value.setter
    def color_value(self, val: str):
        _val = val.translate(BAD_CHARS)
        self._color_value = _val
        self.state_str = self._control_hdr_str + f"{_val}\n"



"""clock.py

Copyright (c) 2022, DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import datetime
import logging
import threading
import time

import zmq
from astral import LocationInfo
from astral.sun import sun
from dateutil import tz

from ..constants import TASK_PULL, CONNECTION_PUB_URL
from .action_station_service_config import ActionServiceCFG, FloatParameterSpec


def make_clock_config(num_tests):
    """Make a timer config"""
    provisioning_list = [
        FloatParameterSpec("Latitude", -90.0, 90.0, "degs", -43.5256),
        FloatParameterSpec("Longitude", -180.0, 180.0, "degs", 172.6398),
    ]
    parameter_list = []

    clock_cfg = ActionServiceCFG(
        "CLK",
        "Local Clock",
        "Send local time and SunUp or SunDown every minute.",
        "CLK1",
        num_tests,
        True,
        True,
        provisioning_list,
        parameter_list
        #  parameter_list_out
    )
    return clock_cfg.__json__()


class ClockThread(threading.Thread):
    """Watch keeping thread returns every minute"""
    def __init__(self, callback) -> None:
        threading.Thread.__init__(self, daemon=True)
        self.running = True
        self.callback = callback
        self.logger_interval = 60
        self.start()

    def close(self):
        """Stop the thread"""
        self.running = False

    def run(self):
        while self.running:
            tstamp = datetime.datetime.now()
            tstamp = tstamp.replace(tzinfo=tz.tzlocal())
            self.callback(tstamp)
            seconds_left = tstamp.second + tstamp.microsecond / 1000000.0
            _, sleep_time = divmod(seconds_left, self.logger_interval)
            sleep_time = self.logger_interval - sleep_time
            time.sleep(sleep_time)


class ClockService(threading.Thread):
    """Action Station Template Class"""

    def clock_message(self, tstamp: datetime.datetime):
        """Send the message"""
        if tstamp.day != self.day:
            self.sun_time = sun(self.location_info.observer, date=tstamp)
            self.day = tstamp.day
        daylight = "SunDown"
        if self.sun_time['sunrise'] < tstamp < self.sun_time['sunset']:
            daylight = "SunUp"
        msg = f"{self.control_msg}\t{daylight}\t{tstamp.year}\t{tstamp.month}\t{tstamp.day}\t{tstamp.hour}\t{tstamp.minute}\n"
        self.send_message(out_message=msg)

    def send_message(self, out_message=""):
        """Send the message"""
        logging.debug("Clock TX: %s", out_message)
        self.task_sender.send_multipart([b"ALL", out_message.encode('utf-8')])

    def close(self):
        """Close the thread"""
        self.clock.close()
        self.running = False

    def __init__(self, device_id: str, action_station_id: str, control_config_dict: dict, context: zmq.Context) -> None:
        threading.Thread.__init__(self, daemon=True)

        self.context = context
        self.running = True

        self.control_id = control_config_dict['controlID']
        self.name = control_config_dict['name']
        self.control_type = control_config_dict['objectType']
        provision_list = control_config_dict['provisioning']

        self.control_msg = f"\t{device_id}\t{self.control_type}\t{self.control_id}"

        latitude = provision_list[0]['value']
        longitude = provision_list[1]['value']
        self.location_info = LocationInfo('name', 'region', 'timezone/name', latitude, longitude)
        tstamp = datetime.datetime.now()
        self.day = tstamp.day
        self.sun_time = sun(self.location_info.observer, date=datetime.datetime.now(tz=tz.tzlocal()))

        self.push_url = TASK_PULL.format(id=action_station_id)

        self.sub_url = CONNECTION_PUB_URL.format(id=action_station_id)

        self.task_sender = self.context.socket(zmq.PUSH)
        self.task_sender.connect(self.push_url)
        self.clock = ClockThread(self.clock_message)
        logging.debug("Clock Started")
        self.start()

    def run(self):
        receiver = self.context.socket(zmq.SUB)
        receiver.connect(self.sub_url)
        receiver.setsockopt_string(zmq.SUBSCRIBE, self.control_msg)
        poller = zmq.Poller()
        poller.register(receiver, zmq.POLLIN)

        while self.running:
            try:
                socks = dict(poller.poll(15))
            except zmq.error.ContextTerminated:
                break
            if receiver in socks:
                message, _ = receiver.recv()
                if message:
                    self.clock_message(datetime.datetime.now())

        self.task_sender.close()
        receiver.close()



import json
import unittest

from dashio import Color, Dial, DialNumberPosition, DialStyle, Precision


class TestDial(unittest.TestCase):

    def _get_cfg_dict(self, cfg_list: list):
        json_str = cfg_list[0].rpartition('\t')[2]
        return json.loads(json_str)

    def test_dial(self):
        test_control = Dial("DIALID")
        test_control.dial_value = 1
        self.assertEqual(test_control.get_state(), '\t{device_id}\tDIAL\tDIALID\t1\n', "Should be 1")

    def test_dial_min(self):
        test_control = Dial("DIALID", dial_min=1)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['min'], 1, "CFG min should be 1")

    def test_dial_max(self):
        test_control = Dial("DIALID", dial_max=1)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['max'], 1, "CFG min should be 1")

    def test_dial_red_value(self):
        test_control = Dial("DIALID", red_value=50)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['redValue'], 50, "CFG redValue should be 50")

    def test_dial_show_min_max(self):
        test_control = Dial("DIALID", show_min_max=True)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['showMinMax'], True, "CFG showMinMax should be True")

    def test_dial_dial_fill_color(self):
        test_control = Dial("DIALID", dial_fill_color=Color.WHEAT)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['dialFillColor'], '110', "CFG dialFillColor should be '110'")

    def test_dial_pointer_color(self):
        test_control = Dial("DIALID", pointer_color=Color.AZURE)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['pointerColor'], '136', "CFG pointerColor should be '136'")

    def test_dial_number_position(self):
        test_control = Dial("DIALID", number_position=DialNumberPosition.LEFT)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['numberPosition'], 'left', "CFG numberPosition should be 'left'")

    def test_dial_style(self):
        test_control = Dial("DIALID", style=DialStyle.BAR)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['style'], 'bar', "CFG style should be 'bar'")

    def test_dial_precision(self):
        test_control = Dial("DIALID", precision=Precision.FIVE)
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['precision'], 5, "CFG precision should be 5")

    def test_dial_units(self):
        test_control = Dial("DIALID", units='Watts')
        cfg_dict = self._get_cfg_dict(test_control.get_cfg(["DEVICEID", "CONTROLID", "DASHID", 1]))
        self.assertEqual(cfg_dict['units'], 'Watts', "CFG units should be 'Watts'")


if __name__ == '__main__':
    unittest.main()



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from ..constants import BAD_CHARS
from .button import Button
from .control import Control, ControlPosition, ControlConfig, _get_icon, _get_title_position
from .enums import Icon, TitlePosition


class ButtonGroupConfig(ControlConfig):
    """ButtonGroupConfig"""
    def __init__(
        self,
        control_id: str,
        title: str,
        text: str,
        icon: Icon,
        grid_view: bool,
        control_position: ControlPosition,
        title_position: TitlePosition
    ) -> None:
        super().__init__(control_id, title, control_position, title_position)
        self.cfg["text"] = text.translate(BAD_CHARS)
        self.cfg["iconName"] = icon.value
        self.cfg["gridView"] = grid_view

    @classmethod
    def from_dict(cls, cfg_dict: dict):
        """Instantiates ButtonGroupConfig from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        ButtonGroupConfig
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            cfg_dict["text"],
            _get_icon(cfg_dict["iconName"]),
            cfg_dict["gridView"],
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"]),
            _get_title_position(cfg_dict["titlePosition"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls


class ButtonGroup(Control):
    """ButtonGroup control that shows a popup of buttons.

    Attributes
    ----------
    control_id : str
        a unique identity string. The identity string must be a unique string for each ButtonGroup per device
    title: str
        A short title for the button group
    text : str
        The text that appears on the ButtonGroup

    Methods
    -------
    add_button(Button)
        Add a button to the ButtonGroup
    """

    def __init__(
        self,
        control_id,
        title="A Button Group",
        text="A Button group with Text",
        title_position=TitlePosition.BOTTOM,
        icon=Icon.MENU,
        grid_view=True,
        control_position=None,
    ):
        """ButtonGroup control that shows a popup of buttons.

        Parameters
        ----------
            control_id : str
                An unique control identity string. The control identity string must be a unique string for each control per device
            title : str, optional:
                [description]. Defaults to "A Button Group".
            text (str, optional):
                [description]. Defaults to "A Button group with Text".
            title_position ([type], optional):
                [description]. Defaults to TitlePosition.BOTTOM.
            icon ([type], optional):
                [description]. Defaults to Icon.MENU.
            grid_view (bool, optional):
                [description]. Defaults to True.
            control_position ([type], optional):
                [description]. Defaults to None.
        """
        super().__init__("BTGP", control_id)
        self._cfg_columnar.append(ButtonGroupConfig(control_id, title, text, icon, grid_view, control_position, title_position))

    @classmethod
    def from_cfg_dict(cls, cfg_dict: dict):
        """Instatiates ButtonGroup from cfg dictionary

        Parameters
        ----------
        cfg_dict : dict
            A dictionary usually loaded from a config json from IoTDashboard App

        Returns
        -------
        ButtonGroup
        """
        tmp_cls = cls(
            cfg_dict["controlID"],
            cfg_dict["title"],
            cfg_dict["text"],
            _get_title_position(cfg_dict["titlePosition"]),
            _get_icon(cfg_dict["iconName"]),
            cfg_dict["gridView"],
            ControlPosition(cfg_dict["xPositionRatio"], cfg_dict["yPositionRatio"], cfg_dict["widthRatio"], cfg_dict["heightRatio"])
        )
        tmp_cls.parent_id = cfg_dict["parentID"]
        return tmp_cls

    def add_button(self, control):
        """[summary]

        Parameters
        ----------
        control : [type]
            [description]

        Raises
        ------
        TypeError
            [description]
        """
        if isinstance(control, Button):
            control.parent_id = self.control_id
        else:
            raise TypeError("Only buttons are allowed")



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
import json
import logging
import threading
import time
import shortuuid
import zmq

from .constants import CONNECTION_PUB_URL, BAD_CHARS
from .action_station import ActionStation
from .iotcontrol.alarm import Alarm
from .iotcontrol.device_view import DeviceView
from .iotcontrol.enums import ControlName
from .load_config import encode_cfg64
from .load_config import CONTROL_INSTANCE_DICT, CONFIG_INSTANCE_DICT


class Device(threading.Thread):
    """Dashio Device

    Attributes
    ----------
    device_type : str
        The type of device (e.g. TempSensor, GPS...).
    device_id : str
        A unique identifier for this device
    device_name : str
        The name for this device (E.g. GlassHouse, MothersZimmerFrame...)

    Methods
    -------
    add_control(iot_control) :
        Add a control to the device.

    remove_control(iot_control) :
        Remove a control from the device.

    get_control(control_type: str, control_id: str):
        returns the instance of a controla leaded into the device.

    add_all_c64_controls(c64_dict: dict):
        Adds all controls defined in c64_dict into the device.

    set_wifi_callback(callback) :
        Set a callback function that is called when the DashIO app provides wifi provisioning information.

    unset_wifi_callback() :
        Clears the wifi callback.

    set_dashio_callback(callback) :
        Set a callback function that is called when the DashIO app provides Dash server provisioning information.

    unset_dashio_callback() :
        Clears the set Dash callback.

    set_name_callback(callback) :
        Set a callback function that is called when the DashIO app provides Name provisioning information.

    unset_name_callback() :
        Clears the set Name callback.

    set_tcp_callback(callback) :
         Set a callback function that is called when the DashIO app provides TCP provisioning information.

    unset_tcp_callback() :
        Clears the set TCP callback.

    set_mqtt_callback(callback) :
        Set a callback function that is called when the DashIO app provides MQTT provisioning information.

    unset_mqtt_callback() :
        Clears the set MQTT callback.
    """

    def _on_message(self, payload):
        data = str(payload, "utf-8").strip()
        command_array = data.split("\n")
        reply = ""
        for command in command_array:
            try:
                reply += self._on_command(command.strip())
            except TypeError:
                pass
        return reply

    def _on_command(self, data):
        data_array = data.split("\t")
        rx_device_id = data_array[0]
        if rx_device_id == "WHO":
            if 'cfgRev' in self._cfg:
                return self._device_id_str + f"\tWHO\t{self.device_type}\t{self.device_name}\t{self._cfg['cfgRev']}\n"
            return self._device_id_str + f"\tWHO\t{self.device_type}\t{self.device_name}\n"
        if rx_device_id != self.device_id:
            return ""
        try:
            cntrl_type = data_array[1]
        except KeyError:
            return ""
        if cntrl_type in self._device_commands_dict:
            return self._device_commands_dict[cntrl_type](data_array)
        try:
            reply = self.controls_dict[cntrl_type + "\t" + data_array[2]]._message_rx_event(data_array)
            if reply:
                return reply.replace("{device_id}", self.device_id)
        except (KeyError, IndexError):
            pass
        return ""

    def _make_connect(self, _):
        return self._device_id_str + "\tCONNECT\n"

    def _make_status(self, _):
        reply = f"\t{self.device_id}\tNAME\t{self._device_name}\n"
        for value in self.controls_dict.values():
            try:
                reply += value.get_state().format(device_id=self.device_id)
            except (TypeError, KeyError):
                pass
        return reply

    def _make_cfg64(self, data):
        try:
            dashboard_id = data[2]
            #  no_views = data[3]
        except IndexError:
            return ""
        reply = self._device_id_str + f"\tCFG\t{dashboard_id}\tC64\t"
        cfg = {}
        cfg["CFG"] = self._cfg
        for control in self.controls_dict.values():
            if control.cntrl_type == "ALM":
                continue
            if control.cntrl_type in ("BLE"):
                cfg[control.cntrl_type] = control.get_cfg64(data)
                continue
            if control.cntrl_type not in cfg:
                cfg[control.cntrl_type] = []
            cfg[control.cntrl_type].extend(control.get_cfg64(data))
        c64_json = encode_cfg64(cfg)
        reply += c64_json + "\n"
        return reply

    def _make_cfg(self, data):
        try:
            dashboard_id = data[2]
            #  no_views = data[3]
        except IndexError:
            return ""
        reply = self._device_id_str + f"\tCFG\t{dashboard_id}\tDVCE\t{json.dumps(self._cfg)}\n"
        dvvw_str = ""
        for control in self.controls_dict.values():
            if control.cntrl_type == "ALM":
                continue
            if control.cntrl_type == "DVVW":
                cfg_list = control.get_cfg(data)
                for cfg in cfg_list:
                    dvvw_str += self._device_id_str + cfg
            else:
                cfg_list = control.get_cfg(data)
                #  logging.debug("CFG: %s", cfg_list)
                for cfg in cfg_list:
                    reply += self._device_id_str + cfg
        reply += dvvw_str
        return reply

    def _send_alarm(self, alarm_id, message_header, message_body):
        payload = self._device_id_str + f"\tALM\t{alarm_id}\t{message_header}\t{message_body}\n"
        logging.debug("ALARM: %s", payload)
        self.tx_zmq_pub.send_multipart([b"ALL", payload.encode('utf-8')])

    def _send_data(self, data: str):
        if not data:
            return
        if isinstance(data, str):
            reply_send = data.format(device_id=self.device_id)
        elif isinstance(data, list):
            reply_send = "\t" + "\t".join(data) + "\n"
        try:
            self.tx_zmq_pub.send_multipart([b"ALL", reply_send.encode('utf-8')])
        except zmq.error.ZMQError:
            pass

    def _send_announce(self):
        payload = self._device_id_str + f"\tWHO\t{self.device_type}\t{self.device_name}\n"
        logging.debug("ANNOUNCE: %s", payload)
        self.tx_zmq_pub.send_multipart([b"DASH", payload.encode('utf-8')])

    def is_control_loaded(self, control_type, control_id) -> bool:
        """Is the control loaded in the device?"""
        key = f"{control_type}\t{control_id}"
        return key in self.controls_dict

    def add_all_c64_controls(self, c64_dict):
        """Loads all the controls in cfg_dict into the device.

        Parameters
        ----------
        c64_dict : Dict
            dictionary of the CFG loaded by decode_cfg from a CFG64 or json
        """
        for control_type, control_list in c64_dict.items():
            if isinstance(control_list, list):
                for control in control_list:
                    key = f"{control_type}\t{control['controlID']}"
                    if self.is_control_loaded(control_type, control['controlID']):
                        cfg = CONFIG_INSTANCE_DICT[control_type].from_dict(control)
                        self.controls_dict[key].add_config_columnar(cfg)
                    else:
                        new_control = CONTROL_INSTANCE_DICT[control_type].from_cfg_dict(control)
                        self.add_control(new_control)

    def add_control(self, iot_control):
        """Add a control to the device.

        Parameters
        ----------
            iot_control : iotControl
        """
        try:
            if isinstance(iot_control, Alarm):
                iot_control.add_transmit_message_callback(self._send_alarm)
            else:
                iot_control.add_transmit_message_callback(self._send_data)
        except AttributeError:
            pass
        key = f"{iot_control.cntrl_type}\t{iot_control.control_id}"

        if key not in self.controls_dict:
            self.inc_config_revision()
            if isinstance(iot_control, DeviceView):
                self._cfg["numDeviceViews"] += 1
            self.controls_dict[key] = iot_control
            return True
        return False

    def get_control(self, control_type: ControlName, control_id: str):
        """Get the control instance.

        Parameters
        ----------
            control_type : ControlName
                The type of control
            control_id : str
                The controlID of the control
        """
        key = f"{control_type.value}\t{control_id}"
        return self.controls_dict.get(key, None)

    def remove_control(self, iot_control):
        """Remove a control a control to the device.

        Parameters
        ----------
            iot_control : iotControl
        """
        if isinstance(iot_control, DeviceView):
            self._cfg["numDeviceViews"] -= 1
        key = f"{iot_control.cntrl_type}\t{iot_control.control_id}"
        if key in self.controls_dict:
            del self.controls_dict[key]

    def _set_devicesetup(self, control_name: str, settable: bool):
        if settable:
            self._device_commands_dict[control_name.upper()] = getattr(self, '_' + control_name + '_rx_event', None)
            if control_name not in self._device_setup_list:
                self._device_setup_list.append(control_name)
        else:
            try:
                del self._device_commands_dict[control_name.upper()]
            except KeyError:
                pass
            try:
                self._device_setup_list.remove(control_name)
            except ValueError:
                pass
        self._cfg["deviceSetup"] = ','.join(self._device_setup_list)

    def _add_action_device_setup(self, settable: bool):
        if settable:
            if 'actn' not in self._device_setup_list:
                self._device_setup_list.append('actn')
        else:
            try:
                del self._device_commands_dict['actn']
            except KeyError:
                pass
            try:
                self._device_setup_list.remove('actn')
            except ValueError:
                pass
        self._cfg["deviceSetup"] = ','.join(self._device_setup_list)

    def set_wifi_callback(self, callback):
        """
        Specify a callback function to be called when IoTDashboard sets wifi parameters.

        Parameters
        ----------
            callback:
                The callback function. It will be invoked with one argument, the msg from IoTDashboard.
                The callback must return a Boolean indicating success.
        """
        self._set_devicesetup("wifi", True)
        self._wifi_rx_callback = callback

    def unset_wifi_callback(self):
        """
        Unset the wifi_rx_callback.
        """
        self._set_devicesetup("wifi", False)
        self._wifi_rx_callback = None

    def _wifi_rx_event(self, msg):
        if self._wifi_rx_callback(msg):
            data = self._device_id_str + "\tWIFI\n"
            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
        return ""

    def set_dashio_callback(self, callback):
        """
        Specify a callback function to be called when IoTDashboard sets dashio parameters.

        Parameters
        ----------
            callback:
                The callback function. It will be invoked with one argument, the msg from IoTDashboard.
                The callback must return a Boolean indicating success.
        """
        self._set_devicesetup("dashio", True)
        self._dashio_rx_callback = callback

    def unset_dashio_callback(self):
        """
        Unset the dashio callback function.
        """
        self._set_devicesetup("dashio", False)
        self._dashio_rx_callback = None

    def _dashio_rx_event(self, msg):
        if self._dashio_rx_callback(msg):
            data = self._device_id_str + "\tDASHIO\n"
            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
        return ""

    def set_name_callback(self, callback):
        """
        Specify a callback function to be called when IoTDashboard sets dashio parameters.

        Parameters
        ----------
            callback:
                The callback function. It will be invoked with one argument, the msg from IoTDashboard.
                The callback must return the new name.
        """
        self._set_devicesetup("name", True)
        self._name_rx_callback = callback

    def unset_name_callback(self):
        """
        Unset the name callback function.
        """
        self._set_devicesetup("name", False)
        self._name_rx_callback = None

    def _name_rx_event(self, msg):
        name = self._name_rx_callback(msg)
        if name:
            self._device_name = name
            data = self._device_id_str + f"\tNAME\t{name}\n"
            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
        return ""

    def set_tcp_callback(self, callback):
        """Specify a callback function to be called when IoTDashboard sets tcp parameters.

        Parameters
        ----------
            callback:
                The callback function. It will be invoked with one argument, the msg from IoTDashboard.
                The callback must return a Boolean indicating success.
        """
        self._set_devicesetup("tcp", True)
        self._tcp_rx_callback = callback

    def unset_tcp_callback(self):
        """
        Unset the tcp callback function.
        """
        self._set_devicesetup("tcp", False)
        self._tcp_rx_callback = None

    def _tcp_rx_event(self, msg):
        if self._tcp_rx_callback(msg):
            data = self._device_id_str + "\tTCP\n"
            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
        return ""

    def set_mqtt_callback(self, callback):
        """
        Specify a callback function to be called when IoTDashboard sets mqtt parameters.

        Parameters
        ----------
            callback:
                The callback function. It will be invoked with one argument, the msg from IoTDashboard.
                The callback must return a Boolean indicating success.
        """
        self._set_devicesetup("mqtt", True)
        self._mqtt_rx_callback = callback

    def unset_mqtt_callback(self):
        """
        Unset the mqtt callback function.
        """
        self._set_devicesetup("mqtt", False)
        self._mqtt_rx_callback = None

    def _mqtt_rx_event(self, msg):
        if self._mqtt_rx_callback(msg):
            data = self._device_id_str + "\tMQTT\n"
            self.tx_zmq_pub.send_multipart([b"ALL", data.encode('utf-8')])
        return ""

    def register_connection(self, connection):
        """Connections registered here"""
        if connection.zmq_connection_uuid not in self.connections_list:
            logging.debug("DEVICE REG CONECTION")
            self.connections_list.append(connection.zmq_connection_uuid)
            self.rx_zmq_sub.connect(CONNECTION_PUB_URL.format(id=connection.zmq_connection_uuid))
            connection.rx_zmq_sub.connect(CONNECTION_PUB_URL.format(id=self.zmq_connection_uuid))
            self._send_announce()
        if self._add_actions:
            self.action_station.register_connection(connection)

    def __init__(
        self,
        device_type: str,
        device_id: str,
        device_name: str,
        add_actions: bool = False,
        cfg_dict: dict = None,
        context: zmq.Context = None
    ) -> None:
        """DashDevice

        Parameters
        ----------
            device_type : str
                A Short description of the device type.
            device_id : str
                A unique identifier for this device
            device_name : str
                The name for this device
            cfg_dict : dict optional
                Setup dict to cfgRev and adds controls defined in cfg_dict, defaults None
            add_actions : boolean
                To include actions or not, defaults false
            context : optional
                ZMQ context. Defaults to None.
        """
        threading.Thread.__init__(self, daemon=True)

        self.zmq_connection_uuid = "DVCE:" + shortuuid.uuid()
        self._b_zmq_connection_uuid = self.zmq_connection_uuid.encode()
        self.context = context or zmq.Context.instance()
        self._wifi_rx_callback = None
        self._dashio_rx_callback = None
        self._name_rx_callback = None
        self._tcp_rx_callback = None
        self._mqtt_rx_callback = None
        self.device_type = device_type.translate(BAD_CHARS)
        self.device_id = device_id.translate(BAD_CHARS)
        self._b_device_id = self.device_id.encode('utf-8')
        self._device_name = device_name.strip()
        self._device_setup_list = []
        self.connections_list = []
        self._device_commands_dict = {}
        self._device_commands_dict['CONNECT'] = self._make_connect
        self._device_commands_dict['STATUS'] = self._make_status
        self._device_commands_dict['CFG'] = self._make_cfg
        self.controls_dict = {}
        self._cfg = {}
        self._device_id_str = f"\t{device_id}"
        self._cfg["numDeviceViews"] = 0
        if cfg_dict is not None:
            self._cfg["cfgRev"] = cfg_dict['CFG']['cfgRev']
            self.add_all_c64_controls(cfg_dict)

        self._add_actions = add_actions
        if self._add_actions:
            self._add_action_device_setup(True)
            self.action_station = ActionStation(self, context=self.context)
            time.sleep(0.5)
            self.action_station.device_zmq_sub.connect(CONNECTION_PUB_URL.format(id=self.zmq_connection_uuid))
        self.running = True
        time.sleep(0.5)
        self.start()

    def use_cfg64(self):
        """Generate a CFG64 formated CFG message
        """
        self._device_commands_dict['CFG'] = self._make_cfg64

    def use_cfg(self):
        """Generate JSON formated CFG messages
        """
        self._device_commands_dict['CFG'] = self._make_cfg

    @property
    def number_of_device_views(self) -> int:
        """Number of device views registered for the device

        Returns
        -------
        int
            Number of device views
        """
        return self._cfg["numDeviceViews"]

    @number_of_device_views.setter
    def number_of_device_views(self, val: int):
        self._cfg["numDeviceViews"] = val

    @property
    def config_revision(self) -> int:
        """Sets the cfgRev number. Increment this number to indicate a new CFG

        Returns
        -------
        int
            The cfgRev number
        """
        return self._cfg["cfgRev"]

    @config_revision.setter
    def config_revision(self, val: int):
        self._cfg["cfgRev"] = val

    def inc_config_revision(self):
        """Incements the configuration revision."""
        if "cfgRev" in self._cfg:
            self._cfg["cfgRev"] = self._cfg["cfgRev"] + 1
        else:
            self._cfg["cfgRev"] = 1

    @property
    def device_name(self) -> str:
        """Device name

        Returns
        -------
        str
            The device name
        """
        return self._device_name

    @device_name.setter
    def device_name(self, val: str):
        self._device_name = val
        self._send_data(f"\t{{device_id}}\tNAME\t{self._device_name}\t")

    def close(self):
        """Close the device"""
        if self._add_actions:
            self.action_station.close()
        self.running = False

    def _local_command(self, msg_dict):
        if msg_dict['msgType'] == 'send_announce':
            self._send_announce()

    def run(self):
        # Continue the network loop, exit when an error occurs

        self.tx_zmq_pub = self.context.socket(zmq.PUB)
        self.tx_zmq_pub.bind(CONNECTION_PUB_URL.format(id=self.zmq_connection_uuid))
        self.rx_zmq_sub = self.context.socket(zmq.SUB)
        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"\tWHO")
        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"COMMAND")
        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, self._b_zmq_connection_uuid)
        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "\t" + self.device_id)

        poller = zmq.Poller()
        poller.register(self.rx_zmq_sub, zmq.POLLIN)

        while self.running:
            try:
                socks = dict(poller.poll(50))
            except zmq.error.ContextTerminated:
                break
            if self.rx_zmq_sub in socks:
                try:
                    [data, msg_from] = self.rx_zmq_sub.recv_multipart()
                except ValueError:
                    logging.debug("Device value error")
                #  logging.debug("DEVICE RX: %s ,%s", msg_from, data)
                if data == b"COMMAND":
                    msg_dict = json.loads(msg_from)
                    self._local_command(msg_dict)
                    continue
                reply = self._on_message(data)
                if reply != "":
                    #  logging.debug("DEVICE TX: %s ,%s", msg_from, data)
                    self.tx_zmq_pub.send_multipart([msg_from, reply.encode('utf-8')])
        self.tx_zmq_pub.close()
        self.rx_zmq_sub.close()
        self.context.term()



"""
MIT License

Copyright (c) 2020 DashIO-Connect

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
from enum import Enum


class ControlName(Enum):
    """
    All the control names
    """
    AVD = "AVD"
    DVVW = "DVVW"
    MENU = "MENU"
    BTGP = "BTGP"
    BTTN = "BTTN"
    TEXT = "TEXT"
    GRPH = "GRPH"
    DIAL = "DIAL"
    CLR = "CLR"
    TGRPH = "TGRPH"
    KNOB = "KNOB"
    SLCTR = "SLCTR"
    SLDR = "SLDR"
    DIR = "DIR"
    LOG = "LOG"
    LBL = "LBL"


class Icon(Enum):
    """
    All the Icons
    """
    NONE = "None"
    DOT = "Dot"
    PAD = "Pad"
    UP = "Up"
    DOWN = "Down"
    LEFT = "Left"
    RIGHT = "Right"
    UP_LEFT = "Up Left"
    DOWN_LEFT = "Down Left"
    DOWN_RIGHT = "Down Right"
    UP_RIGHT = "Up Right"
    LEFT_DOUBLE = "Left Double"
    RIGHT_DOUBLE = "Right Double"
    BEGINNING = "Beginning"
    END = "End"
    PAUSE = "Pause"
    STOP = "Stop"
    PLAY = "Play"
    COG = "Cog"
    LIGHT_BULB = "Light Bulb"
    ON_OFF = "On Off"
    REFRESH = "Refresh"
    TRASH = "Trash"
    MUSIC = "Music"
    SOUND = "Sound"
    BELL = "Bell"
    ATTENTION_1 = "Attention 1"
    ATTENTION_2 = "Attention 2"
    CIRCLE = "Circle"
    SQUARE = "Square"
    TRIANGLE = "Triangle"
    TICK = "Tick"
    CROSS = "Cross"
    MAIL = "Mail"
    BOAT = "Boat"
    SAILBOAT = "Sailboat"
    CAR = "Car"
    BRIGHTNESS = "Brightness"
    TEMPERATURE = "Temperature"
    CLOUD = "Cloud"
    WEATHER = "Weather"
    LIGHTNING = "Lightning"
    DOOROPEN = "Door Open"
    DOORCLOSED = "Door Closed"
    BEDROOM = "Bedroom"
    GARAGE = "Garage"
    HOUSE = "House"
    LED = "LED"
    PADLOCK = "Padlock"
    PROPELLER_1 = "Propeller 1"
    PROPELLER_2 = "Propeller 2"
    FAN = "Fan"
    MENU = "Menu"
    ZOOM_IN = "Zoom In"
    ZOOM_OUT = "Zoom Out"
    RIGHT_DOUBLE_2 = "Right Double 2"
    LOCATION = "Location"
    DIRECTION = "Direction"
    SWITCH = "Switch"


class SoundName(Enum):
    """Sound names"""
    DEFAULT = "Default"
    QUIET = "Quiet"
    BELL = "Bell"
    DOORBELL1 = "Door Bell 1"
    DOORBELL2 = "Door Bell 2"
    ALARM = "Alarm"
    COW = "Cow"
    OVERSPEED = "Overspeed"
    SADTRUMPED = "Sad Trumped"
    SHIPHORN = "Ship Horn"
    SQUEAKY = "Squeaky"
    SLAP = "Slap"
    WHIP1 = "Whip 1"
    WHIP2 = "Whip 2"
    PLOP = "Plop"
    PLOPPLIPPLIP = "Plop Plip Plip"
    SWITCH = "Switch"
    WHOOSH = "Whoosh"
    BOING = "Boing"


class DeviceViewStyle(Enum):
    """DeciveView Styles"""
    BASIC = "BASIC"
    BORDERS = "BORDERS"


class ColorPickerStyle(Enum):
    """Picker styles"""
    WHEEL = "Wheel"
    SPECTRUM = "Spectrum"


class LabelStyle(Enum):
    """Label styles"""
    BASIC = "BASIC"
    GROUP = "GROUP"
    BORDER = "BORDER"


class KnobStyle(Enum):
    """Knob styles"""
    NORMAL = "NORMAL"
    PAN = "PAN"


class Precision(Enum):
    """Precisions"""
    OFF = 0
    ONE = 1
    TWO = 2
    THREE = 3
    FOUR = 4
    FIVE = 5
    SIX = 6


class TitlePosition(Enum):
    """Title positions"""
    NONE = "None"
    TOP = "Top"
    BOTTOM = "Bottom"


class ButtonState(Enum):
    """Button states"""
    ON = "ON"
    OFF = "OFF"
    FLASH = "FLASH"


class Keyboard(Enum):
    """Keyboards"""
    NONE = "NONE"
    ALL = "ALL"
    NUMERIC = "NUM"
    DATE = "DATE"
    TIME = "TIME"
    DATETIME = "DATETIME"
    INTERVAL = "INTVL"
    INT = "INT"
    HEX = "HEX"


class TextFormat(Enum):
    """Text formats"""
    NONE = "NONE"
    NUMBER = "NUM"
    DATETIME = "DATETIME"
    DATELONG = "DTLONG"
    INTERVAL = "INTVL"


class TextAlignment(Enum):
    """Text alignments"""
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class SliderBarStyle(Enum):
    """Slider bar types"""
    SEG = "seg"
    SOLID = "solid"


class DialNumberPosition(Enum):
    """Dial number positions"""
    OFF = "off"
    LEFT = "left"
    RIGHT = "right"
    CENTER = "center"


class DialStyle(Enum):
    """Dial styles"""
    PIE = "pie"
    PIEINV = "pieinv"
    BAR = "bar"


class DirectionStyle(Enum):
    """Direction styles"""
    NSEW = "nsew"
    DEG = "deg"
    DEGPS = "degps"


class ChartLineType(Enum):
    """Graph line types"""
    LINE = "LINE"
    BAR = "BAR"
    SEGBAR = "SEGBAR"
    PEAKBAR = "PEAKBAR"
    DELETE = "DEL"


class ChartXAxisLabelsStyle(Enum):
    """Graph X axis label styles"""
    ON = "on"
    BETWEEN = "between"


class TimeGraphLineType(Enum):
    """Time graph line types"""
    LINE = "LINE"
    BAR = "BAR"
    BOOL = "BOOL"
    DEL = "DEL"


class TimeGraphPositionOfKey(Enum):
    """Time graph positions of key"""
    TOPLEFT = "Top Left"
    TOPRIGHT = "Top Right"


class Color(Enum):
    """Colors"""
    BLACK = 0
    WHITE = 1
    MAROON = 2
    SIENNA = 3
    RED = 4
    ORANGE_RED = 5
    DEEPPINK = 6
    ORANGE = 7
    GOLD = 8
    YELLOW = 9
    GREEN = 10
    SEA_GREEN = 11
    LIME = 12
    LIGHT_GREEN = 13
    SPRING_GREEN = 14
    OLIVE = 15
    MIDNIGHT_BLUE = 16
    NAVY_BLUE = 17
    BLUE = 18
    ROYAL_BLUE = 19
    STEEL_BLUE = 20
    SKY_BLUE = 21
    AQUA = 22
    TURQUOISE = 23
    TEAL = 24
    DARK_SLATE_GRAY = 25
    PURPLE = 26
    SLATE_BLUE = 27
    BLUE_VIOLET = 28
    FUSCIA = 29
    VIOLET = 30
    PINK = 31
    ROSY_BROWN = 32
    BISQUE = 33
    SEA_SHELL = 34
    LAVENDER = 35
    SLATE_GRAY = 36
    DIM_GRAY = 37
    GRAY = 38
    DARK_GRAY = 39
    SILVER = 40
    LIGHT_GRAY = 41
    GAINSBORO = 42
    GHOST_WHITE = 43
    WHITE_SMOKE = 44
    DARK_RED = 45
    BROWN = 46
    FIREBRICK = 47
    CRIMSON = 48
    TOMATO = 49
    CORAL = 50
    INDIAN = 51
    LIGHT_CORAL = 52
    DARK_SALMON = 53
    SALMON = 54
    LIGHT_SALMON = 55
    DARK_ORANGE = 56
    DARK_GOLDEN_ROD = 57
    GOLDEN_ROD = 58
    PALE_GOLDEN_ROD = 59
    DARK_KHAKI = 60
    KHAKI = 61
    YELLOW_GREEN = 62
    DARK_OLIVE_GREEN = 63
    OLIVE_DRAB = 64
    LAWN_GREEN = 65
    CHARTREUSE = 66
    GREEN_YELLOW = 67
    DARK_GREEN = 68
    FOREST_GREEN = 69
    LIME_GREEN = 70
    PALE_GREEN = 71
    DARK_SEA_GREEN = 72
    MEDIUM_SPRING_GREEN = 73
    MEDIUM_AQUAMARINE = 74
    MEDIUM_SEA_GREEN = 75
    LIGHT_SEA_GREEN = 76
    DARK_CYAN = 77
    LIGHT_CYAN = 78
    DARK_TURQUOISE = 79
    MEDIUM_TURQUOISE = 80
    PALE_TURQUOISE = 81
    AQUAMARINE = 82
    POWDER_BLUE = 83
    CADET_BLUE = 84
    CORNFLOWER_BLUE = 85
    DEEP_SKY_BLUE = 86
    DODGER_BLUE = 87
    LIGHT_BLUE = 88
    LIGHT_SKY_BLUE = 89
    DARK_BLUE = 90
    MEDIUM_BLUE = 91
    INDIGO = 92
    DARK_SLATE_BLUE = 93
    MEDIUM_SLATE_BLUE = 94
    MEDIUM_PURPLE = 95
    DARK_MAGENTA = 96
    DARK_VIOLET = 97
    DARK_ORCHID = 98
    MEDIUM_ORCHID = 99
    THISTLE = 100
    PLUM = 101
    ORCHID = 102
    MEDIUM_VIOLET_RED = 103
    PALE_VIOLET_RED = 104
    HOT_PINK = 105
    LIGHT_PINK = 106
    ANTIQUE_WHITE = 107
    BEIGE = 108
    BLANCHED_ALMOND = 109
    WHEAT = 110
    CORN_SILK = 111
    LEMON_CHIFFON = 112
    LIGHT_GOLDEN_ROD_YELLOW = 113
    LIGHT_YELLOW = 114
    SADDLE_BROWN = 115
    CHOCOLATE = 116
    PERU = 117
    SANDY_BROWN = 118
    BURLY_WOOD = 119
    TAN = 120
    MOCCASIN = 121
    NAVAJO_WHITE = 122
    PEACH_PUFF = 123
    MISTY_ROSE = 124
    LAVENDER_BLUSH = 125
    LINEN = 126
    OLD_LACE = 127
    PAPAYA_WHIP = 128
    MINT_CREAM = 129
    LIGHT_SLATE_GRAY = 130
    LIGHT_STEEL_BLUE = 131
    FLORAL_WHITE = 132
    ALICE_BLUE = 133
    HONEYDEW = 134
    IVORY = 135
    AZURE = 136
    SNOW = 137
